# Copyright (c) 2015, MapR Technologies
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#       http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.


import sahara_plugin_mapr.plugins.mapr.domain.configuration_file as bcf
import sahara_plugin_mapr.plugins.mapr.domain.node_process as np
import sahara_plugin_mapr.plugins.mapr.domain.service as s
import sahara_plugin_mapr.plugins.mapr.util.validation_utils as vu


HBASE_MASTER = np.NodeProcess(
    name='hbmaster',
    ui_name='HBase-Master',
    package='mapr-hbase-master',
    open_ports=[60000, 60010]
)
HBASE_REGION_SERVER = np.NodeProcess(
    name='hbregionserver',
    ui_name='HBase-RegionServer',
    package='mapr-hbase-regionserver',
    open_ports=[60020]
)
HBASE_THRIFT = np.NodeProcess(
    name='hbasethrift',
    ui_name='HBase-Thrift',
    package='mapr-hbasethrift',
    open_ports=[9090]
)
HBASE_REST = np.NodeProcess(
    name="hbaserestgateway",
    ui_name="HBase REST",
    package="mapr-hbase-rest",
    open_ports=[8080, 8085],
)


class HBase(s.Service):
    def __init__(self):
        super(HBase, self).__init__()
        self._name = 'hbase'
        self._ui_name = 'HBase'
        self._node_processes = [
            HBASE_MASTER,
            HBASE_REGION_SERVER,
            HBASE_THRIFT,
        ]
        self._cluster_defaults = ['hbase-default.json']
        self._validation_rules = [
            vu.at_least(1, HBASE_MASTER),
            vu.at_least(1, HBASE_REGION_SERVER),
        ]
        self._ui_info = [
            ("HBase Master", HBASE_MASTER, {s.SERVICE_UI: "http://%s:60010"}),
        ]

    def get_config_files(self, cluster_context, configs, instance=None):
        hbase_site = bcf.HadoopXML("hbase-site.xml")
        hbase_site.remote_path = self.conf_dir(cluster_context)
        if instance:
            hbase_site.fetch(instance)
        hbase_site.load_properties(configs)
        return [hbase_site]


class HBaseV094(HBase):
    def __init__(self):
        super(HBaseV094, self).__init__()
        self._version = '0.94.24'
        self._dependencies = [('mapr-hbase', self.version)]


class HBaseV0987(HBase):
    def __init__(self):
        super(HBaseV0987, self).__init__()
        self._version = '0.98.7'
        self._dependencies = [('mapr-hbase', self.version)]


class HBaseV0989(HBase):
    def __init__(self):
        super(HBaseV0989, self).__init__()
        self._version = '0.98.9'
        self._dependencies = [('mapr-hbase', self.version)]
        self._node_processes.append(HBASE_REST)
        self._ui_info.append(
            ("HBase REST", HBASE_REST, {s.SERVICE_UI: "http://%s:8085"}),
        )


class HBaseV09812(HBase):
    def __init__(self):
        super(HBaseV09812, self).__init__()
        self._version = "0.98.12"
        self._dependencies = [("mapr-hbase", self.version)]
        self._node_processes.append(HBASE_REST)
        self._ui_info.append(
            ("HBase REST", HBASE_REST, {s.SERVICE_UI: "http://%s:8085"}),
        )


class HBaseV111(HBase):
    def __init__(self):
        super(HBaseV111, self).__init__()
        self._version = "1.1.1"
        self._dependencies = [("mapr-hbase", self.version)]
        self._node_processes.append(HBASE_REST)
        self._ui_info.append(
            ("HBase REST", HBASE_REST, {s.SERVICE_UI: "http://%s:8085"}),
        )



# Copyright (c) 2015, MapR Technologies
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#       http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.


import itertools
import random
import string

from oslo_log import log as logging

import sahara.plugins.provisioning as p
import sahara.plugins.utils as utils
from sahara_plugin_mapr.i18n import _
import sahara_plugin_mapr.plugins.mapr.domain.configuration_file as bcf
import sahara_plugin_mapr.plugins.mapr.domain.node_process as np
import sahara_plugin_mapr.plugins.mapr.domain.service as s
import sahara_plugin_mapr.plugins.mapr.services.hbase.hbase as hbase
import sahara_plugin_mapr.plugins.mapr.services.hive.hive as hive
import sahara_plugin_mapr.plugins.mapr.services.httpfs.httpfs as httpfs
import sahara_plugin_mapr.plugins.mapr.services.impala.impala as impala
import sahara_plugin_mapr.plugins.mapr.services.mapreduce.mapreduce as mr
import sahara_plugin_mapr.plugins.mapr.services.mysql.mysql as mysql
import sahara_plugin_mapr.plugins.mapr.services.oozie.oozie as oozie
import sahara_plugin_mapr.plugins.mapr.services.sentry.sentry as sentry
import sahara_plugin_mapr.plugins.mapr.services.spark.spark as spark
import sahara_plugin_mapr.plugins.mapr.services.sqoop.sqoop2 as sqoop
import sahara_plugin_mapr.plugins.mapr.services.yarn.yarn as yarn
import sahara_plugin_mapr.plugins.mapr.util.event_log as el
import sahara_plugin_mapr.plugins.mapr.util.general as g
import sahara_plugin_mapr.plugins.mapr.util.password_utils as pu
import sahara_plugin_mapr.plugins.mapr.util.validation_utils as vu

LOG = logging.getLogger(__name__)

HUE = np.NodeProcess(
    name='hue',
    ui_name='Hue',
    package='mapr-hue',
    open_ports=[8002, 8888]
)

HUE_LIVY = np.NodeProcess(
    name="livy",
    ui_name="Hue Livy",
    package="mapr-hue-livy",
    open_ports=[8998]
)


class Hue(s.Service):
    THRIFT_VERSIONS = [5, 7]
    THRIFT_VERSION = p.Config(
        name="Thrift version",
        applicable_target="Hue",
        scope='cluster',
        config_type="dropdown",
        config_values=[(v, v) for v in THRIFT_VERSIONS],
        priority=1,
        description=_(
            'Specifies thrift version.')
    )

    def __init__(self):
        super(Hue, self).__init__()
        self._name = 'hue'
        self._ui_name = 'Hue'
        self._node_processes = [HUE]
        self._ui_info = None
        self._validation_rules = [
            vu.exactly(1, HUE),
            vu.on_same_node(HUE, httpfs.HTTP_FS),
            vu.on_same_node(HUE_LIVY, spark.SPARK_SLAVE),
        ]
        self._priority = 2

    def get_ui_info(self, cluster_context):
        # Hue uses credentials of the administrative user (PAM auth)
        return [('HUE', HUE, {s.SERVICE_UI: 'http://%s:8888',
                              'Username': 'mapr',
                              'Password': pu.get_mapr_password(cluster_context
                                                               .cluster)})]

    def get_configs(self):
        return [Hue.THRIFT_VERSION]

    def conf_dir(self, cluster_context):
        return '%s/desktop/conf' % self.home_dir(cluster_context)

    def get_config_files(self, cluster_context, configs, instance=None):
        template = 'plugins/mapr/services/hue/resources/hue_%s.template'
        # hue.ini
        hue_ini = bcf.TemplateFile("hue.ini")
        hue_ini.remote_path = self.conf_dir(cluster_context)
        hue_ini.parse(utils.get_file_text(template % self.version,
                                          'sahara_plugin_mapr'))
        hue_ini.add_properties(self._get_hue_ini_props(cluster_context))
        hue_ini.add_property("thrift_version",
                             configs[self.THRIFT_VERSION.name])

        # # hue.sh
        hue_sh_template = 'plugins/mapr/services/hue/' \
                          'resources/hue_sh_%s.template'
        hue_sh = bcf.TemplateFile("hue.sh")
        hue_sh.remote_path = self.home_dir(cluster_context) + '/bin'
        hue_sh.parse(utils.get_file_text(hue_sh_template % self.version,
                                         'sahara_plugin_mapr'))
        hue_sh.add_property('hadoop_version', cluster_context.hadoop_version)
        hue_sh.mode = 777

        hue_instances = cluster_context.get_instances(HUE)
        for instance in hue_instances:
            if instance not in cluster_context.changed_instances():
                cluster_context.should_be_restarted[self] += [instance]

        return [hue_ini, hue_sh]

    def _get_hue_ini_props(self, cluster_context):
        db_instance = mysql.MySQL.get_db_instance(cluster_context)
        is_yarn = cluster_context.cluster_mode == 'yarn'
        hue_specs = mysql.MySQL.HUE_SPECS
        rdbms_specs = mysql.MySQL.RDBMS_SPECS

        result = {
            'db_host': db_instance.internal_ip,
            'hue_name': hue_specs.db_name,
            'hue_user': hue_specs.user,
            'hue_password': hue_specs.password,
            'rdbms_name': rdbms_specs.db_name,
            'rdbms_user': rdbms_specs.user,
            'rdbms_password': rdbms_specs.password,
            'resource_manager_uri': cluster_context.resource_manager_uri,
            'yarn_mode': is_yarn,
            'rm_host': cluster_context.get_instance_ip(yarn.RESOURCE_MANAGER),
            'webhdfs_url': cluster_context.get_instance_ip(httpfs.HTTP_FS),
            'jt_host': cluster_context.get_instance_ip(mr.JOB_TRACKER),
            'oozie_host': cluster_context.get_instance_ip(oozie.OOZIE),
            'sqoop_host': cluster_context.get_instance_ip(
                sqoop.SQOOP_2_SERVER),
            'impala_host': cluster_context.get_instance_ip(
                impala.IMPALA_STATE_STORE),
            'zk_hosts_with_port':
                cluster_context.get_zookeeper_nodes_ip_with_port(),
            'secret_key': self._generate_secret_key()
        }

        hive_host = cluster_context.get_instance(hive.HIVE_SERVER_2)
        if hive_host:
            hive_service = cluster_context.get_service(hive.HIVE_SERVER_2)
            result.update({
                'hive_host': hive_host.internal_ip,
                'hive_version': hive_service.version,
                'hive_conf_dir': hive_service.conf_dir(cluster_context),
            })

        hbase_host = cluster_context.get_instance(hbase.HBASE_THRIFT)
        if hbase_host:
            hbase_service = cluster_context.get_service(hbase.HBASE_THRIFT)
            result.update({
                'hbase_host': hbase_host.internal_ip,
                'hbase_conf_dir': hbase_service.conf_dir(cluster_context),
            })

        livy_host = cluster_context.get_instance(HUE_LIVY)
        if livy_host:
            result.update({
                'livy_host': livy_host.internal_ip
            })

        sentry_host = cluster_context.get_instance(sentry.SENTRY)
        if sentry_host:
            ui_name = sentry.Sentry().ui_name
            sentry_version = cluster_context.get_chosen_service_version(
                ui_name)
            sentry_service = cluster_context. \
                _find_service_instance(ui_name, sentry_version)
            result.update({
                'sentry_host': sentry_host.internal_ip,
                'sentry_conf': sentry_service.conf_dir(cluster_context)
            })

        return result

    def post_install(self, cluster_context, instances):
        hue_instance = cluster_context.get_instance(HUE)

        @el.provision_event(name=_("Migrating Hue database"),
                            instance=hue_instance)
        def migrate_database(remote, cluster_context):
            hue_home = self.home_dir(cluster_context)
            cmd = '%(activate)s && %(syncdb)s && %(migrate)s'
            args = {
                'activate': 'source %s/build/env/bin/activate' % hue_home,
                'syncdb': '%s/build/env/bin/hue syncdb --noinput' % hue_home,
                'migrate': '%s/build/env/bin/hue migrate' % hue_home,
            }
            remote.execute_command(cmd % args, run_as_root=True, timeout=600)

        def hue_syncdb_workround(remote):
            cmd = 'printf "/opt/mapr/lib\n$JAVA_HOME/jre/lib/amd64/server\n"' \
                  ' | tee /etc/ld.so.conf.d/mapr-hue.conf && ldconfig'
            remote.execute_command(cmd, run_as_root=True)

        def centos7_workaround(remote):
            cmd = 'ln -s /lib64/libsasl2.so.3.0.0 /lib64/libsasl2.so.2' \
                  ' && rpm -ivh --nodeps http://yum.mariadb.org/5.5.49/' \
                  'rhel7-amd64/rpms/MariaDB-5.5.49-centos7-x86_64-compat.rpm'
            remote.execute_command(cmd, run_as_root=True,
                                   raise_when_error=False)

        with hue_instance.remote() as r:
            LOG.debug("Executing Hue database migration")
            # workaround for centos7
            if cluster_context.distro_version.split('.')[0] == '7':
                centos7_workaround(r)
            # temporary workaround to prevent failure of db migrate on mapr 5.2
            hue_syncdb_workround(r)
            migrate_database(r, cluster_context)
        self._copy_hive_configs(cluster_context, hue_instance)
        self._install_jt_plugin(cluster_context, hue_instance)
        self._set_service_dir_owner(cluster_context, instances)

    def _copy_hive_configs(self, cluster_context, hue_instance):
        hive_server = cluster_context.get_instance(hive.HIVE_SERVER_2)
        if not hive_server or hive_server == hue_instance:
            LOG.debug('No Hive Servers found. Skip')
            return
        hive_service = cluster_context.get_service(hive.HIVE_SERVER_2)
        hive_conf_dir = hive_service.conf_dir(cluster_context)
        g.copy(hive_conf_dir, hive_server, hive_conf_dir, hue_instance, 'root')

    def update(self, cluster_context, instances=None):
        if self._should_restart(cluster_context, instances):
            self.restart(instances)

    def post_start(self, cluster_context, instances):
        self.update(cluster_context, instances=instances)

    def _should_restart(self, cluster_context, instances):
        app_services = [
            impala.Impala(),
            hive.Hive(),
            hbase.HBase(),
            sqoop.Sqoop2(),
            spark.SparkOnYarn(),
        ]
        instances = [
            cluster_context.filter_instances(instances, service=service)
            for service in app_services]
        return bool(g.unique_list(itertools.chain(*instances)))

    def jt_plugin_path(self, cluster_context):
        path = ('%(home)s/desktop/libs/hadoop/java-lib'
                '/hue-plugins-%(version)s-mapr.jar')
        args = {
            'home': self.home_dir(cluster_context),
            'version': self.version,
        }
        return path % args

    @el.provision_event(name="Install Hue Job Tracker plugin",
                        instance_reference=2)
    def _install_jt_plugin(self, cluster_context, hue_instance):
        LOG.debug("Copying Hue JobTracker plugin")
        job_trackers = cluster_context.get_instances(mr.JOB_TRACKER)
        if not job_trackers:
            LOG.debug('No JobTrackers found. Skip')
            return
        jt_plugin_src = self.jt_plugin_path(cluster_context)
        jt_plugin_dest = cluster_context.hadoop_lib + '/jt_plugin.jar'
        for jt in job_trackers:
            g.copy(jt_plugin_src, hue_instance, jt_plugin_dest, jt, 'root')

    def _generate_secret_key(self, length=80):
        ascii_alphanum = string.ascii_letters + string.digits
        generator = random.SystemRandom()
        return ''.join(generator.choice(ascii_alphanum) for _ in range(length))


class HueV360(Hue):
    def __init__(self):
        super(HueV360, self).__init__()
        self._version = '3.6.0'


class HueV370(Hue):
    def __init__(self):
        super(HueV370, self).__init__()
        self._version = '3.7.0'


class HueV381(Hue):
    def __init__(self):
        super(HueV381, self).__init__()
        self._version = "3.8.1"
        self._dependencies = [("mapr-hue-base", self.version)]
        self._node_processes = [HUE, HUE_LIVY]
        self._validation_rules.append(vu.at_most(1, HUE_LIVY))


class HueV390(Hue):
    def __init__(self):
        super(HueV390, self).__init__()
        self._version = "3.9.0"
        self._dependencies = [("mapr-hue-base", self.version)]
        self._node_processes = [HUE, HUE_LIVY]
        self._validation_rules.append(vu.at_most(1, HUE_LIVY))


class HueV310(Hue):
    def __init__(self):
        super(HueV310, self).__init__()
        self._version = "3.10.0"
        self._node_processes = [HUE]
        self._validation_rules.append(vu.at_most(1, HUE_LIVY))


class HueLivyV310(s.Service):
    def __init__(self):
        super(HueLivyV310, self).__init__()
        self._name = 'hue-livy'
        self._ui_name = 'Livy'
        self._version = "3.10.0"
        self._node_processes = [HUE_LIVY]
        self._validation_rules = [vu.at_most(1, HUE_LIVY),
                                  vu.depends_on(HueV310(), self),
                                  vu.depends_on(spark.SparkOnYarnV201(), self),
                                  vu.on_same_node(HUE_LIVY, spark.SPARK_SLAVE)]

    def get_config_files(self, cluster_context, configs, instance=None):
        livy_conf_template = 'plugins/mapr/services/hue/' \
                             'resources/livy_conf_%s.template'
        livy_conf = bcf.TemplateFile("livy.conf")
        livy_conf.parse(utils.get_file_text(livy_conf_template % self.version,
                                            'sahara_plugin_mapr'))
        livy_conf.remote_path = self.home_dir(cluster_context) + '/conf'

        livy_sh_template = 'plugins/mapr/services/hue/' \
                           'resources/livy_sh_%s.template'
        livy_sh = bcf.TemplateFile("livy-env.sh")
        livy_sh.remote_path = self.home_dir(cluster_context) + '/conf'
        livy_sh.parse(utils.get_file_text(livy_sh_template % self.version,
                                          'sahara_plugin_mapr'))
        livy_sh.add_property('hadoop_version', cluster_context.hadoop_version)
        livy_sh.add_property('spark_version', spark.SparkOnYarnV201().version)
        livy_sh.mode = 777

        hue_instances = cluster_context.get_instances(HUE)
        for instance in hue_instances:
            if instance not in cluster_context.changed_instances():
                cluster_context.should_be_restarted[self] += [instance]

        return [livy_sh, livy_conf]

    def post_install(self, cluster_context, instances):
        self._set_service_dir_owner(cluster_context, instances)

    def post_start(self, cluster_context, instances):
        self.restart(instances)



# Copyright (c) 2015, MapR Technologies
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#       http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.


import random

from oslo_log import log as logging
from oslo_serialization import jsonutils as json
from oslo_utils import timeutils

from sahara.plugins import context
import sahara.plugins.exceptions as ex
from sahara.plugins import utils
from sahara_plugin_mapr.i18n import _
import sahara_plugin_mapr.plugins.mapr.abstract.node_manager as s
import sahara_plugin_mapr.plugins.mapr.services.management.management as mng
import sahara_plugin_mapr.plugins.mapr.services.maprfs.maprfs as mfs
import sahara_plugin_mapr.plugins.mapr.util.event_log as el

LOG = logging.getLogger(__name__)

GET_SERVER_ID_CMD = ('maprcli node list -json -filter [ip==%s] -columns id'
                     ' | grep id | grep -o \'[0-9]*\'')
NODE_LIST_CMD = 'maprcli node list -json'
MOVE_NODE_CMD = 'maprcli node move -serverids %s -topology /decommissioned'
REMOVE_NODE_CMD = ('maprcli node remove -filter [ip==%(ip)s] -nodes %(nodes)s'
                   ' -zkconnect %(zookeepers)s')
WAIT_NODE_ALARM_NO_HEARTBEAT = 360

WARDEN_SERVICE = 'warden'
START = 'start'
STOP = 'stop'
DELAY = 5
DEFAULT_RETRY_COUNT = 10


class BaseNodeManager(s.AbstractNodeManager):
    def move_nodes(self, cluster_context, instances):
        LOG.debug("Moving the nodes to /decommissioned topology")
        cldb_instances = self._get_cldb_instances(cluster_context, instances)
        with random.choice(cldb_instances).remote() as cldb_remote:
            for instance in instances:
                with instance.remote() as r:
                    command = GET_SERVER_ID_CMD % instance.internal_ip
                    ec, out = r.execute_command(command, run_as_root=True)
                    command = MOVE_NODE_CMD % out.strip()
                    cldb_remote.execute_command(command, run_as_root=True)
        LOG.info("Nodes successfully moved")

    def remove_nodes(self, cluster_context, instances):
        LOG.debug("Removing nodes from cluster")
        cldb_instances = self._get_cldb_instances(cluster_context, instances)
        with random.choice(cldb_instances).remote() as cldb_remote:
            for instance in instances:
                args = {
                    'ip': instance.internal_ip,
                    'nodes': instance.fqdn(),
                    'zookeepers':
                        cluster_context.get_zookeeper_nodes_ip_with_port(),
                }
                command = REMOVE_NODE_CMD % args
                cldb_remote.execute_command(command, run_as_root=True)
        LOG.info("Nodes successfully removed")

    def start(self, cluster_context, instances=None):
        instances = instances or cluster_context.get_instances()
        zookeepers = cluster_context.filter_instances(instances, mng.ZOOKEEPER)
        cldbs = cluster_context.filter_instances(instances, mfs.CLDB)
        others = [i for i in instances
                  if not cluster_context.check_for_process(i, mfs.CLDB)]
        utils.add_provisioning_step(cluster_context.cluster.id,
                                    _("Start ZooKeepers nodes"),
                                    len(zookeepers))
        self._start_zk_nodes(zookeepers)
        utils.add_provisioning_step(cluster_context.cluster.id,
                                    _("Start CLDB nodes"), len(cldbs))
        self._start_cldb_nodes(cldbs)
        if others:
            utils.add_provisioning_step(cluster_context.cluster.id,
                                        _("Start non-CLDB nodes"),
                                        len(list(others)))
            self._start_non_cldb_nodes(others)
        self._await_cldb(cluster_context, instances)

    def stop(self, cluster_context, instances=None):
        instances = instances or cluster_context.get_instances()
        zookeepers = cluster_context.filter_instances(instances, mng.ZOOKEEPER)
        utils.add_provisioning_step(cluster_context.cluster.id,
                                    _("Stop ZooKeepers nodes"),
                                    len(zookeepers))
        self._stop_zk_nodes(zookeepers)
        utils.add_provisioning_step(cluster_context.cluster.id,
                                    _("Stop Warden nodes"), len(instances))
        self._stop_warden_on_nodes(instances)

    def _await_cldb(self, cluster_context, instances=None, timeout=600):
        instances = instances or cluster_context.get_instances()
        cldb_node = cluster_context.get_instance(mfs.CLDB)
        start_time = timeutils.utcnow()
        retry_count = 0
        with cldb_node.remote() as r:
            LOG.debug("Waiting {count} seconds for CLDB initialization".format(
                count=timeout))
            while timeutils.delta_seconds(start_time,
                                          timeutils.utcnow()) < timeout:
                ec, out = r.execute_command(NODE_LIST_CMD,
                                            raise_when_error=False)
                resp = json.loads(out)
                status = resp['status']
                if str(status).lower() == 'ok':
                    ips = [n['ip'] for n in resp['data']]
                    retry_count += 1
                    for i in instances:
                        if (i.internal_ip not in ips and
                                (retry_count > DEFAULT_RETRY_COUNT)):
                            msg = _("Node failed to connect to CLDB: %s"
                                    ) % i.internal_ip
                            raise ex.HadoopProvisionError(msg)
                    break
                else:
                    context.sleep(DELAY)
            else:
                raise ex.HadoopProvisionError(_("CLDB failed to start"))

    def _start_nodes(self, instances, sys_service):
        with context.PluginsThreadGroup() as tg:
            for instance in instances:
                tg.spawn('start-%s-%s' % (sys_service, instance.id),
                         self._start_service, instance, sys_service)

    def _stop_nodes(self, instances, sys_service):
        with context.PluginsThreadGroup() as tg:
            for instance in instances:
                tg.spawn('stop-%s-%s' % (sys_service, instance.id),
                         self._stop_service, instance, sys_service)

    def _start_zk_nodes(self, instances):
        LOG.debug('Starting ZooKeeper nodes')
        self._start_nodes(instances, mng.ZOOKEEPER.ui_name)
        LOG.info('ZooKeeper nodes successfully started')

    def _start_cldb_nodes(self, instances):
        LOG.debug('Starting CLDB nodes')
        self._start_nodes(instances, WARDEN_SERVICE)
        LOG.info('CLDB nodes successfully started')

    def _start_non_cldb_nodes(self, instances):
        LOG.debug('Starting non-control nodes')
        self._start_nodes(instances, WARDEN_SERVICE)
        LOG.info('Non-control nodes successfully started')

    def _stop_zk_nodes(self, instances):
        self._stop_nodes(instances, mng.ZOOKEEPER.ui_name)

    def _stop_warden_on_nodes(self, instances):
        self._stop_nodes(instances, WARDEN_SERVICE)

    @staticmethod
    def _do_service_action(instance, service, action):
        with instance.remote() as r:
            cmd = "service mapr-%(service)s %(action)s"
            args = {'service': service.lower(), 'action': action}
            cmd = cmd % args
            LOG.debug(
                'Executing "{command}" on node={ip}'.format(
                    command=cmd, ip=instance.internal_ip))
            r.execute_command(cmd, run_as_root=True)

    @el.provision_event(instance_reference=1)
    def _start_service(self, instance, service):
        return self._do_service_action(instance, service, START)

    @el.provision_event(instance_reference=1)
    def _stop_service(self, instance, service):
        return self._do_service_action(instance, service, STOP)

    def _get_cldb_instances(self, cluster_context, instances):
        current = self._get_current_cluster_instances(cluster_context,
                                                      instances)
        return cluster_context.filter_instances(current, mfs.CLDB)

    @staticmethod
    def await_no_heartbeat():
        delay = WAIT_NODE_ALARM_NO_HEARTBEAT
        LOG.debug('Waiting for "NO_HEARBEAT" alarm')
        context.sleep(delay)

    def _get_current_cluster_instances(self, cluster_context, instances):
        all_instances = cluster_context.get_instances()
        return [x for x in all_instances if x not in instances]



# Copyright (c) 2015, MapR Technologies
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#       http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.


from oslo_log import log as logging

from sahara.plugins import context
import sahara.plugins.provisioning as p
from sahara.plugins import utils
from sahara_plugin_mapr.i18n import _
import sahara_plugin_mapr.plugins.mapr.domain.configuration_file as bcf
import sahara_plugin_mapr.plugins.mapr.domain.node_process as np
import sahara_plugin_mapr.plugins.mapr.domain.service as s
import sahara_plugin_mapr.plugins.mapr.util.event_log as el
import sahara_plugin_mapr.plugins.mapr.util.general as g
import sahara_plugin_mapr.plugins.mapr.util.validation_utils as vu

LOG = logging.getLogger(__name__)

CLDB = np.NodeProcess(
    name='cldb',
    ui_name='CLDB',
    package='mapr-cldb',
    open_ports=[7222, 7220, 7221]
)
FILE_SERVER = np.NodeProcess(
    name='fileserver',
    ui_name='FileServer',
    package='mapr-fileserver',
    open_ports=[]
)
NFS = np.NodeProcess(
    name='nfs',
    ui_name='NFS',
    package='mapr-nfs',
    open_ports=[2049, 9997, 9998]
)


class MapRFS(s.Service):
    _CREATE_DISK_LIST = 'plugins/mapr/resources/create_disk_list_file.sh'
    _DISK_SETUP_CMD = '/opt/mapr/server/disksetup -F /tmp/disk.list'
    _DISK_SETUP_TIMEOUT = 600

    ENABLE_MAPR_DB_NAME = 'Enable MapR-DB'
    HEAP_SIZE_PERCENT_NAME = 'MapR-FS heap size percent'

    ENABLE_MAPR_DB_CONFIG = p.Config(
        name=ENABLE_MAPR_DB_NAME,
        applicable_target='general',
        scope='cluster',
        config_type="bool",
        priority=1,
        default_value=True,
        description=_('Specifies that MapR-DB is in use.')
    )

    HEAP_SIZE_PERCENT = p.Config(
        name=HEAP_SIZE_PERCENT_NAME,
        applicable_target='MapRFS',
        scope='cluster',
        config_type="int",
        priority=1,
        default_value=8,
        description=_(
            'Specifies heap size for MapR-FS in percents of maximum value.')
    )

    def __init__(self):
        super(MapRFS, self).__init__()
        self._ui_name = 'MapRFS'
        self._node_processes = [CLDB, FILE_SERVER, NFS]
        self._ui_info = [
            ('Container Location Database (CLDB)', CLDB,
             {s.SERVICE_UI: 'http://%s:7221'}),
        ]
        self._validation_rules = [
            vu.at_least(1, CLDB),
            vu.each_node_has(FILE_SERVER),
            vu.on_same_node(CLDB, FILE_SERVER),
            vu.has_volumes(),
        ]

    def service_dir(self, cluster_context):
        return

    def home_dir(self, cluster_context):
        return

    def conf_dir(self, cluster_context):
        return '%s/conf' % cluster_context.mapr_home

    def post_install(self, cluster_context, instances):
        LOG.debug('Initializing MapR FS')
        instances = instances or cluster_context.get_instances()
        file_servers = cluster_context.filter_instances(instances, FILE_SERVER)
        utils.add_provisioning_step(cluster_context.cluster.id,
                                    _("Initializing MapR-FS"),
                                    len(file_servers))
        with context.PluginsThreadGroup() as tg:
            for instance in file_servers:
                tg.spawn('init-mfs-%s' % instance.id,
                         self._init_mfs_instance, instance)
        LOG.info('MapR FS successfully initialized')

    @el.provision_event(instance_reference=1)
    def _init_mfs_instance(self, instance):
        self._generate_disk_list_file(instance, self._CREATE_DISK_LIST)
        self._execute_disksetup(instance)

    def _generate_disk_list_file(self, instance, path_to_disk_setup_script):
        LOG.debug('Creating disk list file')
        g.run_script(instance, path_to_disk_setup_script, 'root',
                     *instance.storage_paths())

    def _execute_disksetup(self, instance):
        with instance.remote() as rmt:
            rmt.execute_command(
                self._DISK_SETUP_CMD, run_as_root=True,
                timeout=self._DISK_SETUP_TIMEOUT)

    def get_configs(self):
        return [MapRFS.ENABLE_MAPR_DB_CONFIG, MapRFS.HEAP_SIZE_PERCENT]

    def get_config_files(self, cluster_context, configs, instance=None):
        default_path = 'plugins/mapr/services/maprfs/resources/cldb.conf'
        cldb_conf = bcf.PropertiesFile("cldb.conf")
        cldb_conf.remote_path = self.conf_dir(cluster_context)
        if instance:
            cldb_conf.fetch(instance)
        cldb_conf.parse(utils.get_file_text(default_path,
                                            'sahara_plugin_mapr'))
        cldb_conf.add_properties(self._get_cldb_conf_props(cluster_context))

        warden_conf = bcf.PropertiesFile("warden.conf")
        warden_conf.remote_path = "/opt/mapr/conf/"
        if instance:
            warden_conf.fetch(instance)
        warden_conf.add_properties(
            {'service.command.mfs.heapsize.percent': configs[
                self.HEAP_SIZE_PERCENT_NAME]})

        return [cldb_conf, warden_conf]

    def _get_cldb_conf_props(self, cluster_context):
        zookeepers = cluster_context.get_zookeeper_nodes_ip_with_port()
        return {'cldb.zookeeper.servers': zookeepers}



# Copyright (c) 2015, MapR Technologies
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#       http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.


from sahara.plugins import utils


def provision_step(name, cluster_context_reference=1, instances_reference=2):
    def wrapper(function):
        def wrapped(*args, **kwargs):
            cluster_context = _find_argument(
                cluster_context_reference, *args, **kwargs)
            instances = _find_argument(instances_reference, *args, **kwargs)

            cluster_id = cluster_context.cluster.id
            instance_count = len(instances)

            utils.add_provisioning_step(cluster_id, name, instance_count)

            return function(*args, **kwargs)

        return wrapped

    return wrapper


def provision_event(instance_reference=0, name=None, instance=None):
    def wrapper(function):
        def wrapped(*args, **kwargs):
            event_instance = instance or _find_argument(instance_reference,
                                                        *args, **kwargs)
            if name:
                utils.add_provisioning_step(
                    event_instance.node_group.cluster.id, name, 1)
            try:
                result = function(*args, **kwargs)
                utils.add_successful_event(event_instance)
                return result
            except Exception as exception:
                utils.add_fail_event(event_instance, exception)
                raise exception

        return wrapped

    return wrapper


def _find_argument(reference, *args, **kwargs):
    if isinstance(reference, int):
        return args[reference]
    if isinstance(reference, str):
        return kwargs[reference]



# Copyright (c) 2015, MapR Technologies
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#       http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.


import sahara_plugin_mapr.plugins.mapr.base.base_version_handler as bvh
from sahara_plugin_mapr.plugins.mapr.services.drill import drill
from sahara_plugin_mapr.plugins.mapr.services.flume import flume
from sahara_plugin_mapr.plugins.mapr.services.hbase import hbase
from sahara_plugin_mapr.plugins.mapr.services.hive import hive
from sahara_plugin_mapr.plugins.mapr.services.httpfs import httpfs
from sahara_plugin_mapr.plugins.mapr.services.hue import hue
from sahara_plugin_mapr.plugins.mapr.services.impala import impala
from sahara_plugin_mapr.plugins.mapr.services.kafka import kafka
from sahara_plugin_mapr.plugins.mapr.services.mahout import mahout
from sahara_plugin_mapr.plugins.mapr.services.management import management
from sahara_plugin_mapr.plugins.mapr.services.maprfs import maprfs
from sahara_plugin_mapr.plugins.mapr.services.oozie import oozie
from sahara_plugin_mapr.plugins.mapr.services.pig import pig
from sahara_plugin_mapr.plugins.mapr.services.sentry import sentry
from sahara_plugin_mapr.plugins.mapr.services.spark import spark
from sahara_plugin_mapr.plugins.mapr.services.sqoop import sqoop2
from sahara_plugin_mapr.plugins.mapr.services.swift import swift
from sahara_plugin_mapr.plugins.mapr.services.yarn import yarn
import sahara_plugin_mapr.plugins.mapr.versions.v5_2_0_mrv2.context as c

version = "5.2.0.mrv2"


class VersionHandler(bvh.BaseVersionHandler):
    def __init__(self):
        super(VersionHandler, self).__init__()
        self._version = version
        self._required_services = [
            yarn.YARNv270(),
            maprfs.MapRFS(),
            management.Management(),
            oozie.Oozie(),
        ]
        self._services = [
            hive.HiveV12(),
            pig.PigV016(),
            impala.ImpalaV250(),
            flume.FlumeV16(),
            sqoop2.Sqoop2(),
            mahout.MahoutV012(),
            oozie.OozieV420(),
            hue.HueV310(),
            hue.HueLivyV310(),
            hbase.HBaseV111(),
            drill.DrillV19(),
            yarn.YARNv270(),
            maprfs.MapRFS(),
            management.Management(),
            httpfs.HttpFS(),
            swift.Swift(),
            sentry.SentryV16(),
            spark.SparkOnYarnV201(),
            kafka.Kafka(),
            kafka.KafkaConnect(),
            kafka.KafkaRest(),
        ]

    def get_context(self, cluster, added=None, removed=None):
        return c.Context(cluster, self, added, removed)



# Copyright (c) 2015, MapR Technologies
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#       http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.


import sahara_plugin_mapr.plugins.mapr.domain.node_process as np
import sahara_plugin_mapr.plugins.mapr.domain.service as s
import sahara_plugin_mapr.plugins.mapr.util.validation_utils as vu

KAFKA = np.NodeProcess(
    name='kafka',
    ui_name='Kafka',
    package='mapr-kafka',
    open_ports=[9092]
)

KAFKA_REST = np.NodeProcess(
    name='kafka',
    ui_name='Kafka Rest',
    package='mapr-kafka-rest',
    open_ports=[8082]
)

KAFKA_CONNECT_HDFS = np.NodeProcess(
    name='kafka',
    ui_name='Kafka Connect HDFS',
    package='mapr-kafka-connect-hdfs'
)

KAFKA_CONNECT_JDBC = np.NodeProcess(
    name='kafka',
    ui_name='Kafka Connect JDBC',
    package='mapr-kafka-connect-jdbc'
)


class Kafka(s.Service):
    def __init__(self):
        super(Kafka, self).__init__()
        self._version = '0.9.0'
        self._name = 'kafka'
        self._ui_name = 'Kafka'
        self._node_processes = [KAFKA]


class KafkaRest(s.Service):
    def __init__(self):
        super(KafkaRest, self).__init__()
        self._version = '2.0.1'
        self._name = 'kafka-eco'
        self._ui_name = 'Kafka Rest'
        self._node_processes = [KAFKA_REST]
        self._validation_rules = [vu.at_least(1, KAFKA)]


class KafkaConnect(s.Service):
    def __init__(self):
        super(KafkaConnect, self).__init__()
        self._version = '2.0.1'
        self._name = 'kafka-connect'
        self._ui_name = 'Kafka Connect'
        self._node_processes = [KAFKA_CONNECT_HDFS, KAFKA_CONNECT_JDBC]
        self._validation_rules = [vu.at_least(1, KAFKA)]



# Copyright (c) 2015, MapR Technologies
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#       http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.


import sahara_plugin_mapr.plugins.mapr.domain.node_process as np
import sahara_plugin_mapr.plugins.mapr.domain.service as s
import sahara_plugin_mapr.plugins.mapr.util.validation_utils as vu

MAHOUT = np.NodeProcess(
    name='mahout',
    ui_name='Mahout',
    package='mapr-mahout',
    open_ports=[]
)


class Mahout(s.Service):
    def __init__(self):
        super(Mahout, self).__init__()
        self._name = 'mahout'
        self._ui_name = 'Mahout'
        self._node_processes = [MAHOUT]
        self._validation_rules = [vu.at_least(1, MAHOUT)]


class MahoutV09(Mahout):
    def __init__(self):
        super(MahoutV09, self).__init__()
        self._version = '0.9'


class MahoutV010(Mahout):
    def __init__(self):
        super(MahoutV010, self).__init__()
        self._version = '0.10.0'


class MahoutV011(Mahout):
    def __init__(self):
        super(MahoutV011, self).__init__()
        self._version = '0.11.0'


class MahoutV012(Mahout):
    def __init__(self):
        super(MahoutV012, self).__init__()
        self._version = '0.12.0'



# Copyright (c) 2015, MapR Technologies
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#       http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.


import sahara_plugin_mapr.plugins.mapr.abstract.cluster_validator as v
import sahara_plugin_mapr.plugins.mapr.util.validation_utils as vu
import sahara_plugin_mapr.plugins.mapr.versions.version_handler_factory as vhf


class BaseValidator(v.AbstractValidator):
    def validate(self, cluster_context):
        for service in cluster_context.required_services:
            vu.assert_present(service, cluster_context)
        for service in cluster_context.cluster_services:
            for rule in service.validation_rules:
                rule(cluster_context)

    def validate_scaling(self, cluster_context, existing, additional):
        cluster = cluster_context.cluster
        version = cluster.hadoop_version
        handler = vhf.VersionHandlerFactory.get().get_handler(version)
        cluster = vu.create_fake_cluster(cluster, existing, additional)
        cluster_context = handler.get_context(cluster)
        self.validate(cluster_context)



# Copyright (c) 2015, MapR Technologies
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

import six

from sahara.plugins import provisioning as p
from sahara.plugins import testutils as tu
import sahara_plugin_mapr.plugins.mapr.abstract.cluster_context as a_context
from sahara_plugin_mapr.plugins.mapr.domain import service as bs
from sahara_plugin_mapr.plugins.mapr.services.management import management
from sahara_plugin_mapr.plugins.mapr.services.maprfs import maprfs
from sahara_plugin_mapr.plugins.mapr.services.oozie import oozie
from sahara_plugin_mapr.plugins.mapr.services.yarn import yarn
from sahara_plugin_mapr.plugins.mapr.versions.v5_2_0_mrv2 import \
    version_handler
from sahara_plugin_mapr.tests.unit import base as b


class TestHandler(b.SaharaTestCase):
    def __init__(self, *args, **kwds):
        super(TestHandler, self).__init__(*args, **kwds)
        self.handler = version_handler.VersionHandler()
        self.cluster = self._get_cluster()

    def _get_cluster(self):
        i1 = tu.make_inst_dict('id_1', 'instance_1', '1.1.1.1')
        master_proc = [
            yarn.RESOURCE_MANAGER.ui_name,
            yarn.NODE_MANAGER.ui_name,
            yarn.HISTORY_SERVER.ui_name,
            maprfs.CLDB.ui_name,
            maprfs.FILE_SERVER.ui_name,
            oozie.OOZIE.ui_name,
            management.ZOOKEEPER.ui_name,
        ]

        master_ng = tu.make_ng_dict('master', 'large', master_proc, 1, [i1])
        cluster_configs = {
            'Service': {
                'key': 'value',
                'Service Version': '1.1',
            },
        }

        cluster = tu.create_cluster(
            name='test_cluster',
            project='large',
            plugin='mapr',
            version='5.2.0.mrv2',
            node_groups=[master_ng],
            cluster_configs=cluster_configs,
        )
        self.ng = cluster.node_groups[0]
        self.instance = self.ng.instances[0]
        return cluster

    def test_get_context(self):
        self.assertIsInstance(self.handler.get_context(self._get_cluster()),
                              a_context.AbstractClusterContext)

    def test_get_services(self):
        services = self.handler.get_services()
        for service in services:
            self.assertIsInstance(service, bs.Service)

    def test_get_np_dict(self):
        np_dict = self.handler.get_node_processes()
        self.assertIsNotNone(np_dict)
        self.assertIsInstance(np_dict, dict)
        for k, v in six.iteritems(np_dict):
            self.assertIsInstance(k, six.string_types)
            self.assertIsInstance(v, list)
            self.assertNotEqual(0, len(v))

    def test_get_configs(self):
        configs = self.handler.get_configs()
        self.assertIsInstance(configs, list)
        for config in configs:
            self.assertIsInstance(config, p.Config)

    def test_get_version_configs(self):
        version_configs = self.handler._get_version_configs()
        self.assertIsInstance(version_configs, list)
        for conf in version_configs:
            self.assertIsInstance(conf, p.Config)
            self.assertNotEqual(0, len(conf.config_values))
            self.assertEqual('dropdown', conf.config_type)

    def test_get_configs_dict(self):
        configs_dict = self.handler.get_configs_dict()
        self.assertIsInstance(configs_dict, dict)
        for k, v in six.iteritems(configs_dict):
            self.assertIsInstance(k, six.string_types)
            self.assertIsInstance(v, dict)

    def test_get_open_ports(self):
        ports_list = self.handler.get_open_ports(self.ng)
        self.assertIsNotNone(ports_list)
        self.assertIsInstance(ports_list, list)
        for port in ports_list:
            self.assertIsInstance(port, int)



# Copyright (c) 2015, MapR Technologies
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#       http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.


class Distro(object):
    def __init__(self, name, internal_name, install_cmd, version_separator):
        self._name = name
        self._internal_name = internal_name
        self._install_command = install_cmd
        self._version_separator = version_separator

    @property
    def name(self):
        return self._name

    @property
    def internal_name(self):
        return self._internal_name

    @property
    def install_command(self):
        return self._install_command

    @property
    def version_separator(self):
        return self._version_separator

    def create_install_cmd(self, packages):
        s = self.version_separator

        def join_package_version(pv_item):
            p, v = pv_item if len(pv_item) > 1 else (pv_item[0], None)
            return p + s + v + '*' if v else p

        packages = ' '.join(map(join_package_version, packages))
        command = '%(install_cmd)s %(packages)s'
        args = {'install_cmd': self.install_command, 'packages': packages}
        return command % args


UBUNTU = Distro(
    name='Ubuntu',
    internal_name='Ubuntu',
    install_cmd='apt-get install --force-yes -y',
    version_separator='=',
)

CENTOS = Distro(
    name='CentOS',
    internal_name='CentOS',
    install_cmd='yum install -y',
    version_separator='-',
)

RHEL = Distro(
    name='RedHatEnterpriseServer',
    internal_name='RedHat',
    install_cmd='yum install -y',
    version_separator='-',
)

SUSE = Distro(
    name='Suse',
    internal_name='Suse',
    install_cmd='zypper',
    version_separator=':',
)


def get_all():
    return [UBUNTU, CENTOS, RHEL, SUSE]


def get(instance):
    with instance.remote() as r:
        name = r.get_os_distrib()
        for d in get_all():
            if d.internal_name.lower() in name:
                return d


def get_version(instance):
    with instance.remote() as r:
        version = r.get_os_version()
        return version



# Copyright (c) 2015, MapR Technologies
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.


from oslo_utils import uuidutils

from sahara.plugins import context
from sahara.plugins import objects
import sahara.plugins.utils as utils


def unique_list(iterable, mapper=lambda i: i):
    result = []

    for item in iterable:
        value = mapper(item)
        if value not in result:
            result.append(value)

    return result


def _run_as(user, command):
    if not user:
        return command
    return 'sudo -u %(user)s %(command)s' % {'user': user, 'command': command}


def unique_file_name(base='/tmp'):
    return '%(base)s/%(uuid)s' % {'base': base,
                                  'uuid': uuidutils.generate_uuid()}


def remove(instance, path, recursive=True, run_as=None):
    with instance.remote() as r:
        args = {'recursive': '-r' if recursive else '', 'path': path}
        r.execute_command(_run_as(run_as, 'rm %(recursive)s %(path)s' % args))


def create_archive(instance, path, archive=None, run_as=None):
    if not archive:
        archive = unique_file_name('/tmp')
    args = {'archive': archive, 'path': path}
    tar = 'tar cf %(archive)s -C %(path)s .' % args
    with instance.remote() as r:
        r.execute_command(_run_as(run_as, tar))
    return archive


def unpack_archive(instance, src, dest, cleanup=False, run_as=None):
    with instance.remote() as r:
        r.execute_command(_run_as(run_as, 'mkdir -p %s' % dest))
        untar = 'tar xf %(src)s -C %(dest)s' % {'src': src, 'dest': dest}
        r.execute_command(_run_as(run_as, untar))
        if cleanup:
            r.execute_command(_run_as(run_as, 'rm -r %s' % src))


def copy_file(s_path, s_instance, d_path, d_instance, run_as=None, owner=None):
    with s_instance.remote() as sr:
        data = sr.read_file_from(s_path, run_as_root=(run_as == 'root'))
    with d_instance.remote() as dr:
        write_file(dr, d_path, data, owner=owner)


def copy_dir(s_path, s_instance, d_path, d_instance, run_as=None):
    s_path = create_archive(s_instance, s_path, run_as=run_as)
    tmp_path = unique_file_name('/tmp')
    copy_file(s_path, s_instance, tmp_path, d_instance, run_as)
    unpack_archive(d_instance, tmp_path, d_path, True, run_as)
    remove(s_instance, s_path, True, run_as)


def copy(s_path, s_instance, d_path, d_instance, run_as=None):
    if is_directory(s_instance, s_path):
        copy_dir(s_path, s_instance, d_path, d_instance, run_as)
    else:
        copy_file(s_path, s_instance, d_path, d_instance, run_as)


def run_script(instance, script, run_as=None, *args, **kwargs):
    with instance.remote() as r:
        path = '/tmp/%s.sh' % uuidutils.generate_uuid()
        script = utils.get_file_text(script, 'sahara_plugin_mapr') % kwargs
        r.write_file_to(path, script, run_as_root=(run_as == 'root'))
        r.execute_command(_run_as(run_as, 'chmod +x %s' % path))
        r.execute_command(_run_as(run_as, '%s %s' % (path, ' '.join(args))),
                          timeout=3600)
        # FIXME(aosadchyi): reuse existing remote
        remove(instance, path, run_as=run_as)


def execute_on_instances(instances, function, *args, **kwargs):
    with context.PluginsThreadGroup() as tg:
        for instance in instances:
            t_name = '%s-execution' % function.__name__
            tg.spawn(t_name, function, instance, *args, **kwargs)


def _replace(args, position, value):
    return args[:position] + (value,) + args[position + 1:]


def remote_command(position):
    def wrap(func):
        def wrapped(*args, **kwargs):
            target = args[position]
            if objects.is_object_instance(target):
                with target.remote() as remote:
                    return func(*_replace(args, position, remote), **kwargs)
            return func(*args, **kwargs)

        return wrapped

    return wrap


def execute_command(instances, command, run_as=None):
    def _execute_command(instance):
        with instance.remote() as remote:
            remote.execute_command(_run_as(run_as, command), timeout=1800)

    execute_on_instances(instances, _execute_command)


@remote_command(0)
def is_directory(remote, path):
    command = '[ -d %s ]' % path
    ec = remote.execute_command(command, True, raise_when_error=False)[0]
    return not ec


@remote_command(0)
def chown(remote, owner, path):
    args = {'owner': owner, 'path': path}
    remote.execute_command('chown -R %(owner)s %(path)s' % args, True)


@remote_command(0)
def chmod(remote, mode, path):
    args = {'mode': mode, 'path': path}
    remote.execute_command('chmod -R %(mode)s %(path)s' % args, True)


@remote_command(0)
def mkdir(remote, path, mode=None, owner=''):
    args = {'mode': '-m %s' % mode if mode else '', 'path': path}
    remote.execute_command('mkdir -p %(mode)s %(path)s' % args, bool(owner))
    if owner:
        chown(remote, owner, path)


@remote_command(0)
def write_file(remote, path, data, mode=None, owner=''):
    remote.write_file_to(path, data, run_as_root=bool(owner))
    if mode:
        chmod(remote, mode, path)
    if owner:
        chown(remote, owner, path)


@remote_command(0)
def install_ssh_key(remote, user, private_key, public_key):
    ssh_dir = '/home/%s/.ssh' % user
    owner = '%s:%s' % (user, user)
    if not is_directory(remote, ssh_dir):
        mkdir(remote, ssh_dir, 700, owner)
    write_file(remote, '%s/id_rsa.pub' % ssh_dir, public_key, 644, owner)
    write_file(remote, '%s/id_rsa' % ssh_dir, private_key, 600, owner)


@remote_command(0)
def authorize_key(remote, user, public_key):
    authorized_keys = '/home/%s/.ssh/authorized_keys' % user
    remote.append_to_file(authorized_keys, public_key, run_as_root=True)


@remote_command(0)
def download(remote, url, output=None, run_as=None):
    args = {'url': url, 'output': '-O %s' % output if output else ''}
    command = _run_as(run_as, 'wget -q %(output)s %(url)s' % args)
    remote.execute_command(command, timeout=600)


class Singleton(type):
    instances = {}

    def __call__(cls, *args, **kwargs):
        if cls not in cls.instances:
            new_instance = super(Singleton, cls).__call__(*args, **kwargs)
            cls.instances[cls] = new_instance

        return cls.instances[cls]



# Copyright (c) 2015, MapR Technologies
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#       http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.


import sahara_plugin_mapr.plugins.mapr.domain.node_process as np
import sahara_plugin_mapr.plugins.mapr.domain.service as s
import sahara_plugin_mapr.plugins.mapr.util.commands as cmd
import sahara_plugin_mapr.plugins.mapr.util.validation_utils as vu

DRILL = np.NodeProcess(
    name='drill-bits',
    ui_name='Drill',
    package='mapr-drill',
    open_ports=[8047]
)

DRILL_YARN = np.NodeProcess(
    name='drill-bits-yarn',
    ui_name='Drill on YARN',
    package='mapr-drill-yarn',
    open_ports=[8047]
)


class Drill(s.Service):
    def __init__(self):
        super(Drill, self).__init__()
        self._name = 'drill'
        self._ui_name = 'Drill'
        self._node_processes = [DRILL]
        self._ui_info = [('Drill', DRILL, {s.SERVICE_UI: 'http://%s:8047'})]
        self._validation_rules = [
            vu.node_client_package_conflict_vr([DRILL], DRILL_YARN)
        ]

    def install(self, cluster_context, instances):
        # Drill requires running cluster
        pass

    def post_start(self, cluster_context, instances):
        instances = instances or cluster_context.get_instances(DRILL)
        super(Drill, self).install(cluster_context, instances)
        self._set_service_dir_owner(cluster_context, instances)
        for instance in instances:
            cmd.re_configure_sh(instance, cluster_context)


class DrillV07(Drill):
    def __init__(self):
        super(DrillV07, self).__init__()
        self._version = '0.7'


class DrillV08(Drill):
    def __init__(self):
        super(DrillV08, self).__init__()
        self._version = '0.8'


class DrillV09(Drill):
    def __init__(self):
        super(DrillV09, self).__init__()
        self._version = '0.9'


class DrillV11(Drill):
    def __init__(self):
        super(DrillV11, self).__init__()
        self._version = "1.1"


class DrillV12(Drill):
    def __init__(self):
        super(DrillV12, self).__init__()
        self._version = "1.2"


class DrillV14(Drill):
    def __init__(self):
        super(DrillV14, self).__init__()
        self._version = "1.4"


class DrillV16(Drill):
    def __init__(self):
        super(DrillV16, self).__init__()
        self._version = "1.6"


class DrillV18(Drill):
    def __init__(self):
        super(DrillV18, self).__init__()
        self._version = "1.8"
        self._node_processes = [DRILL, DRILL_YARN]


class DrillV19(Drill):
    def __init__(self):
        super(DrillV19, self).__init__()
        self._version = "1.8"
        self._node_processes = [DRILL, DRILL_YARN]



# Copyright (c) 2015, MapR Technologies
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#       http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

from sahara.plugins import swift_helper
from sahara.plugins import topology_helper as topo
import sahara_plugin_mapr.plugins.mapr.domain.configuration_file as bcf
import sahara_plugin_mapr.plugins.mapr.domain.node_process as np
import sahara_plugin_mapr.plugins.mapr.domain.service as s
import sahara_plugin_mapr.plugins.mapr.util.validation_utils as vu

JOB_TRACKER = np.NodeProcess(
    name='jobtracker',
    ui_name='JobTracker',
    package='mapr-jobtracker',
    open_ports=[9001, 50030]
)
TASK_TRACKER = np.NodeProcess(
    name='tasktracker',
    ui_name='TaskTracker',
    package='mapr-tasktracker',
    open_ports=[50060]
)


class MapReduce(s.Service):
    cluster_mode = 'classic'

    def __init__(self):
        super(MapReduce, self).__init__()
        self._ui_name = 'MapReduce'
        self._name = 'hadoop'
        self._version = '0.20.2'
        self._node_processes = [JOB_TRACKER, TASK_TRACKER]
        self._ui_info = [
            ('JobTracker', JOB_TRACKER, {s.SERVICE_UI: 'http://%s:50030'}),
            ('TaskTracker', TASK_TRACKER, {s.SERVICE_UI: 'http://%s:50060'}),
        ]
        self._validation_rules = [
            vu.at_least(1, JOB_TRACKER),
            vu.at_least(1, TASK_TRACKER),
        ]
        self._cluster_defaults = ['mapreduce-cluster.json']

    def _get_packages(self, cluster_context, node_processes):
        result = []

        result += self.dependencies
        result += [(np.package, None) for np in node_processes]

        return result

    # mapred-site.xml
    def get_config_files(self, cluster_context, configs, instance=None):
        core_site = bcf.HadoopXML("core-site.xml")
        core_site.remote_path = self.conf_dir(cluster_context)
        if instance:
            core_site.fetch(instance)
        core_site.add_properties(self._get_core_site_props(cluster_context))

        mapred_site = bcf.HadoopXML("mapred-site.xml")
        mapred_site.remote_path = self.conf_dir(cluster_context)
        if instance:
            mapred_site.fetch(instance)
        mapred_site.load_properties(configs)
        mapred_site.add_properties(
            self._get_mapred_site_props(cluster_context))

        return [core_site, mapred_site]

    def _get_core_site_props(self, cluster_context):
        result = {}
        if cluster_context.is_node_aware:
            result.update(self._get_core_site_node_aware_props())
        for conf in swift_helper.get_swift_configs():
            result[conf['name']] = conf['value']
        for conf in self._get_impersonation_props():
            result[conf['name']] = conf['value']
        return result

    def _get_core_site_node_aware_props(self):
        result = topo.vm_awareness_core_config()
        result = {c['name']: c['value'] for c in result}
        result.update({
            'net.topology.script.file.name': '/opt/mapr/topology.sh',
            'net.topology.script.number.args': '75',
        })
        return result

    def _get_mapred_site_props(self, cluster_context):
        result = {}
        if cluster_context.is_node_aware:
            result.update(self._get_mapred_site_node_aware_props())
        result.update({
            'jobtracker.thrift.address': '0.0.0.0:9290',
            'mapred.jobtracker.plugins':
                'org.apache.hadoop.thriftfs.ThriftJobTrackerPlugin',
        })
        return result

    def _get_mapred_site_node_aware_props(self):
        result = topo.vm_awareness_mapred_config()
        result = {c['name']: c['value'] for c in result}
        # This config causes failure
        result.pop('mapred.task.cache.levels')
        result['mapreduce.jobtracker.taskcache.levels'] = '3'
        return result

    def _get_impersonation_props(self):
        return [
            {'name': 'hadoop.proxyuser.mapr.groups', 'value': '*'},
            {'name': 'hadoop.proxyuser.mapr.hosts', 'value': '*'}
        ]

    def get_file_path(self, file_name):
        template = 'plugins/mapr/services/mapreduce/resources/%s'
        return template % file_name



# Copyright (c) 2015, MapR Technologies
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#       http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.


import abc

import six


@six.add_metaclass(abc.ABCMeta)
class AbstractVersionHandler(object):
    @abc.abstractmethod
    def get_node_processes(self):
        return

    @abc.abstractmethod
    def get_configs(self):
        return

    @abc.abstractmethod
    def configure_cluster(self, cluster):
        pass

    @abc.abstractmethod
    def start_cluster(self, cluster):
        pass

    @abc.abstractmethod
    def validate(self, cluster):
        pass

    @abc.abstractmethod
    def validate_scaling(self, cluster, existing, additional):
        pass

    @abc.abstractmethod
    def scale_cluster(self, cluster, instances):
        pass

    @abc.abstractmethod
    def decommission_nodes(self, cluster, instances):
        pass

    @abc.abstractmethod
    def get_edp_engine(self, cluster, job_type):
        return

    @abc.abstractmethod
    def get_edp_job_types(self):
        return []

    @abc.abstractmethod
    def get_edp_config_hints(self, job_type):
        return {}

    @abc.abstractmethod
    def get_context(self, cluster, added=None, removed=None):
        return

    @abc.abstractmethod
    def get_services(self):
        return

    @abc.abstractmethod
    def get_required_services(self):
        return

    @abc.abstractmethod
    def get_open_ports(self, node_group):
        return



# Copyright (c) 2015, MapR Technologies
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#       http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.


import functools

from sahara.plugins import health_check_base
from sahara_plugin_mapr.i18n import _
import sahara_plugin_mapr.plugins.mapr.abstract.health_checker as hc
from sahara_plugin_mapr.plugins.mapr.domain import node_process as np
from sahara_plugin_mapr.plugins.mapr.services.management import management
from sahara_plugin_mapr.plugins.mapr.services.spark import spark


class BaseHealthChecker(hc.AbstractHealthChecker):
    def _is_available(self, process):
        return process.open_ports and process not in spark.SparkOnYarn().\
            node_processes

    def get_checks(self, cluster_context, instances=None):
        checks = [
            functools.partial(ZookeeperCheck, cluster_context=cluster_context)]
        for node_process in cluster_context.get_node_processes():
            if self._is_available(
                    node_process) and node_process.ui_name != 'ZooKeeper':
                checks.append(functools.partial
                              (MapRNodeProcessCheck,
                               cluster_context=cluster_context,
                               process=node_process))
        return checks


class ZookeeperCheck(health_check_base.BasicHealthCheck):
    def __init__(self, cluster, cluster_context):
        super(ZookeeperCheck, self).__init__(cluster)
        self.cluster_context = cluster_context

    def get_health_check_name(self):
        return 'MapR ZooKeeper check'

    def is_available(self):
        return self.cluster_context.cluster.plugin_name == 'mapr'

    def _is_zookeeper_running(self, instance):
        cmd = 'service mapr-zookeeper status'
        with instance.remote() as r:
            __, out = r.execute_command(cmd, run_as_root=True)
            return 'zookeeper running as process' in out \
                   or 'active (running)' in out

    def check_health(self):
        instances = self.cluster_context.get_instances(
            node_process=management.ZOOKEEPER)
        active_count = 0
        for instance in instances:
            if self._is_zookeeper_running(instance):
                active_count += 1

        if active_count == 0:
            raise health_check_base.RedHealthError(_(
                "ZooKeeper is not in running state"))

        if active_count < len(instances):
            raise health_check_base.YellowHealthError(_(
                "Some ZooKeeper processes are not in running state"))
        return _("ZooKeeper is in running state")


class MapRNodeProcessCheck(health_check_base.BasicHealthCheck):
    IMPORTANT_PROCESSES = [
        'CLDB',
        'FileServer',
        'NodeManager',
        'ResourceManager'
    ]

    def __init__(self, cluster, cluster_context, process):
        super(MapRNodeProcessCheck, self).__init__(cluster)
        self.process = process
        self.cluster_context = cluster_context

    def get_health_check_name(self):
        return 'MapR %s check' % self.process.ui_name

    def is_available(self):
        return self.cluster_context.cluster.plugin_name == 'mapr'

    def check_health(self):
        instances = self.cluster_context.get_instances(
            node_process=self.process)
        active_count = 0
        for instance in instances:
            status = self.process.status(instance)
            if status == np.Status.RUNNING:
                active_count += 1

        if active_count == 0:
            if self.process.ui_name in self.IMPORTANT_PROCESSES:
                raise health_check_base.RedHealthError(_(
                    "%s is not in running state") % self.process.ui_name)
            else:
                raise health_check_base.YellowHealthError(_(
                    "%s is not in running state") % self.process.ui_name)

        if active_count < len(instances):
            if self.process.ui_name in self.IMPORTANT_PROCESSES:
                raise health_check_base.YellowHealthError(_(
                    "Some %s processes are not in running state")
                    % self.process.ui_name)
        return _("%s is in running state") % self.process.ui_name



# Copyright (c) 2015, MapR Technologies
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#       http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.


import six

from sahara_plugin_mapr.i18n import _


def get_node_process_name(node_process):
    # This import is placed here to avoid circular imports
    from sahara_plugin_mapr.plugins.mapr.domain import node_process as np  # noqa

    if isinstance(node_process, np.NodeProcess):
        return node_process.ui_name
    if isinstance(node_process, six.string_types):
        return node_process

    raise TypeError(_("Invalid argument type %s") % type(node_process))


def has_node_process(instance, node_process):
    node_process_name = get_node_process_name(node_process)
    instance_node_processes = instance.node_group.node_processes
    return node_process_name in instance_node_processes


def has_service(instance, service):
    return any(has_node_process(instance, node_process)
               for node_process in service.node_processes)


def filter_by_node_process(instances, node_process):
    return [instance for instance in instances
            if has_node_process(instance, node_process)]


def filter_by_service(instances, service):
    return [instance for instance in instances
            if has_service(instance, service)]



# Copyright (c) 2015, MapR Technologies
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#       http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

from oslo_log import log as logging

import sahara.plugins.context as context
from sahara_plugin_mapr.i18n import _
import sahara_plugin_mapr.plugins.mapr.domain.configuration_file as bcf
import sahara_plugin_mapr.plugins.mapr.domain.node_process as np
import sahara_plugin_mapr.plugins.mapr.domain.service as s
import sahara_plugin_mapr.plugins.mapr.services.mysql.mysql as mysql
import sahara_plugin_mapr.plugins.mapr.util.event_log as el
import sahara_plugin_mapr.plugins.mapr.util.general as g
import sahara_plugin_mapr.plugins.mapr.util.validation_utils as vu

LOG = logging.getLogger(__name__)
OOZIE_START_DELAY = 30

OOZIE = np.NodeProcess(
    name='oozie',
    ui_name='Oozie',
    package='mapr-oozie',
    open_ports=[11000]
)


class Oozie(s.Service):
    def __init__(self):
        super(Oozie, self).__init__()
        self._name = 'oozie'
        self._ui_name = 'Oozie'
        self._node_processes = [OOZIE]
        self._cluster_defaults = ['oozie-default.json']
        self._validation_rules = [vu.exactly(1, OOZIE)]
        self._ui_info = [('Oozie', OOZIE,
                          {s.SERVICE_UI: 'http://%s:11000/oozie'})]

    def libext_path(self):
        return '/opt/mapr/oozie/oozie-%s/oozie-server/lib/' % self.version

    def get_config_files(self, cluster_context, configs, instance=None):
        oozie_site = bcf.HadoopXML("oozie-site.xml")
        oozie_site.remote_path = self.conf_dir(cluster_context)
        if instance:
            oozie_site.fetch(instance)
        oozie_site.load_properties(configs)
        oozie_site.add_properties(self._get_oozie_site_props(cluster_context))
        return [oozie_site]

    def _get_oozie_site_props(self, cluster_context):
        oozie_specs = mysql.MySQL.OOZIE_SPECS

        return {
            'oozie.db.schema.name': oozie_specs.db_name,
            'oozie.service.JPAService.create.db.schema': True,
            'oozie.service.JPAService.jdbc.driver': mysql.MySQL.DRIVER_CLASS,
            'oozie.service.JPAService.jdbc.url': self._get_jdbc_uri(
                cluster_context),
            'oozie.service.JPAService.jdbc.username': oozie_specs.user,
            'oozie.service.JPAService.jdbc.password': oozie_specs.password,
            'oozie.service.HadoopAccessorService.hadoop.configurations':
                '*=%s' % cluster_context.hadoop_conf
        }

    def _get_jdbc_uri(self, cluster_context):
        jdbc_uri = ('jdbc:mysql://%(db_host)s:%(db_port)s/%(db_name)s?'
                    'createDatabaseIfNotExist=true')
        jdbc_args = {
            'db_host': mysql.MySQL.get_db_instance(
                cluster_context).internal_ip,
            'db_port': mysql.MySQL.MYSQL_SERVER_PORT,
            'db_name': mysql.MySQL.OOZIE_SPECS.db_name,
        }
        return jdbc_uri % jdbc_args

    def install(self, cluster_context, instances):
        # oozie requires executed configure.sh
        pass

    def post_configure(self, cluster_context, instances):
        super(Oozie, self).install(cluster_context, instances)
        oozie_instances = cluster_context.filter_instances(instances,
                                                           service=self)
        for instance in oozie_instances:
            with instance.remote() as r:
                r.execute_command(
                    'sudo cp '
                    '/opt/mapr/oozie/oozie-%s/conf/warden.oozie.conf '
                    '/opt/mapr/conf/conf.d' % self.version)

    def post_install(self, cluster_context, instances):
        oozie_inst = cluster_context.get_instance(OOZIE)
        oozie_service = cluster_context.get_service(OOZIE)

        if oozie_service:
            symlink_cmd = (
                'cp /opt/mapr/lib/mysql-connector-*.jar %s' %
                self.libext_path())
            with oozie_inst.remote() as r:
                LOG.debug('Installing MySQL connector for Oozie')
                r.execute_command(symlink_cmd, run_as_root=True,
                                  raise_when_error=False)
        self._set_service_dir_owner(cluster_context, instances)

    def post_start(self, cluster_context, instances):
        instances = cluster_context.filter_instances(instances, OOZIE)
        self._rebuild(cluster_context, instances)

    @el.provision_event(instance_reference=1)
    @g.remote_command(1)
    def _rebuild_oozie_war(self, remote, cluster_context):
        cmd = 'cp -f /opt/mapr-repository/ext-2.2.zip ' \
              '%(home)s/libext/ext-2.2.zip &&' \
              ' %(home)s/bin/oozie-setup.sh -hadoop %(version)s' \
              ' /opt/mapr/hadoop/hadoop-%(version)s'
        args = {'home': self.home_dir(cluster_context),
                'version': cluster_context.hadoop_version}
        remote.execute_command(cmd % args, run_as_root=True)

    def update(self, cluster_context, instances=None):
        instances = instances or cluster_context.get_instances()
        instances = cluster_context.filter_instances(instances, OOZIE)
        self._rebuild(cluster_context, instances)

    @el.provision_step(_("Rebuilt Oozie war"))
    def _rebuild(self, cluster_context, instances):
        OOZIE.stop(list(filter(OOZIE.is_started, instances)))
        g.execute_on_instances(
            instances, self._rebuild_oozie_war, cluster_context)
        OOZIE.start(instances)
        context.sleep(OOZIE_START_DELAY)


class OozieV401(Oozie):
    def __init__(self):
        super(OozieV401, self).__init__()
        self._version = '4.0.1'
        self._dependencies = [('mapr-oozie-internal', self.version)]


class OozieV410(Oozie):
    def __init__(self):
        super(OozieV410, self).__init__()
        self._version = '4.1.0'
        self._dependencies = [('mapr-oozie-internal', self.version)]


class OozieV420(Oozie):
    def __init__(self):
        super(OozieV420, self).__init__()
        self._version = '4.2.0'
        self._dependencies = [('mapr-oozie-internal', self.version)]

    def libext_path(self):
        return '/opt/mapr/oozie/oozie-%s/libext/' % self.version

    def post_install(self, cluster_context, instances):
        super(OozieV420, self).post_install(cluster_context, instances)



# Copyright (c) 2015, MapR Technologies
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

import testtools

import sahara.plugins.exceptions as e
from sahara.plugins import provisioning as p
from sahara.plugins import testutils as tu
from sahara_plugin_mapr.plugins.mapr.domain import node_process as np
from sahara_plugin_mapr.plugins.mapr.services.management import management
from sahara_plugin_mapr.plugins.mapr.services.maprfs import maprfs
from sahara_plugin_mapr.plugins.mapr.services.oozie import oozie
from sahara_plugin_mapr.plugins.mapr.services.swift import swift
from sahara_plugin_mapr.plugins.mapr.services.yarn import yarn
import sahara_plugin_mapr.plugins.mapr.versions.v5_2_0_mrv2.context as cc
from sahara_plugin_mapr.plugins.mapr.versions.v5_2_0_mrv2 import \
    version_handler
from sahara_plugin_mapr.tests.unit import base as b


MANAGEMENT_IP = '1.1.1.1'
INTERNAL_IP = '1.1.1.2'


class TestClusterContext(b.SaharaTestCase):
    def __init__(self, *args, **kwds):
        super(TestClusterContext, self).__init__(*args, **kwds)
        self.fake_np = np.NodeProcess('fake', 'foo', 'bar')

    def _get_context(self):
        i1 = tu.make_inst_dict('id_1', 'instance_1', MANAGEMENT_IP)
        i1['internal_ip'] = INTERNAL_IP
        master_proc = [
            yarn.RESOURCE_MANAGER.ui_name,
            yarn.NODE_MANAGER.ui_name,
            yarn.HISTORY_SERVER.ui_name,
            maprfs.CLDB.ui_name,
            maprfs.FILE_SERVER.ui_name,
            oozie.OOZIE.ui_name,
            management.ZOOKEEPER.ui_name,
        ]
        master_ng = tu.make_ng_dict('master', 'large', master_proc, 1, [i1])
        cluster_configs = {
            'Service': {
                'key': 'value',
                'Service Version': '1.1',
            },
            'Oozie': {
                'Oozie Version': '4.2.0',
            }
        }
        cluster = tu.create_cluster(
            name='test_cluster',
            project='large',
            plugin='mapr',
            version='5.2.0.mrv2',
            node_groups=[master_ng],
            cluster_configs=cluster_configs,
        )
        self.ng = cluster.node_groups[0]
        self.instance = self.ng.instances[0]
        return cc.Context(cluster, version_handler.VersionHandler())

    def test_get_oozie_server_uri(self):
        ctx = self._get_context()
        expected = 'http://%s:11000/oozie' % MANAGEMENT_IP
        self.assertEqual(expected, ctx.oozie_server_uri)

    def test_oozie_server(self):
        ctx = self._get_context()
        node_processes = ctx.oozie_server.node_group.node_processes
        self.assertIn(oozie.OOZIE.ui_name, node_processes)

    def test_oozie_http(self):
        ctx = self._get_context()
        expected = '%s:11000' % MANAGEMENT_IP
        self.assertEqual(expected, ctx.oozie_http)

    def test_configure_sh(self):
        ctx = self._get_context()
        conf_sh = ctx.configure_sh
        pattern = (r'^(\S+)\s+(-N (\S+))\s+(-C (\S+))\s+(-Z (\S+))\s+'
                   r'(-no-autostart)\s+(-f)\s+\s(-HS (\S+))')
        self.assertRegex(conf_sh, pattern)
        self.assertIn('/opt/mapr/server/configure.sh', conf_sh)
        self.assertIn('-C %s' % INTERNAL_IP, conf_sh)
        self.assertIn('-Z %s' % INTERNAL_IP, conf_sh)
        self.assertIn('-HS %s' % INTERNAL_IP, conf_sh)
        self.assertIn('-no-autostart', conf_sh)
        self.assertIn('-N ' + ctx.cluster.name, conf_sh)

    def test_get_cluster_config_value(self):
        ctx = self._get_context()
        conf = p.Config('key', 'Service', 'cluster')
        self.assertEqual('value', ctx._get_cluster_config_value(conf))
        not_set = p.Config('nonset', 'Service', 'cluster')
        self.assertIsNone(ctx._get_cluster_config_value(not_set))

    def test_get_instances(self):
        ctx = self._get_context()
        instances = ctx.get_instances()
        self.assertEqual(1, len(instances))
        rms1 = ctx.get_instances(yarn.RESOURCE_MANAGER)
        self.assertEqual(1, len(rms1))
        rms2 = ctx.get_instances(yarn.RESOURCE_MANAGER.ui_name)
        self.assertEqual(1, len(rms2))
        not_existing_1 = ctx.get_instances(self.fake_np)
        self.assertEqual(0, len(not_existing_1))
        not_existing_2 = ctx.get_instances(self.fake_np.ui_name)
        self.assertEqual(0, len(not_existing_2))

    def test_get_instance(self):
        ctx = self._get_context()
        instance_1 = ctx.get_instance(yarn.RESOURCE_MANAGER)
        self.assertIn(yarn.RESOURCE_MANAGER.ui_name,
                      instance_1.node_group.node_processes)
        instance_2 = ctx.get_instance(yarn.RESOURCE_MANAGER)
        self.assertIn(yarn.RESOURCE_MANAGER.ui_name,
                      instance_2.node_group.node_processes)
        self.assertIsNone(ctx.get_instance(self.fake_np))

    def test_get_instances_ip(self):
        ctx = self._get_context()
        ip_list_1 = ctx.get_instances_ip(yarn.RESOURCE_MANAGER)
        self.assertEqual(1, len(ip_list_1))
        self.assertIn(INTERNAL_IP, ip_list_1)
        ip_list_2 = ctx.get_instances_ip(yarn.RESOURCE_MANAGER.ui_name)
        self.assertEqual(1, len(ip_list_2))
        self.assertIn(INTERNAL_IP, ip_list_2)
        empty_list = ctx.get_instances_ip(self.fake_np)
        self.assertEqual(0, len(empty_list))

    def test_get_instance_ip(self):
        ctx = self._get_context()
        ip_1 = ctx.get_instance_ip(yarn.RESOURCE_MANAGER)
        self.assertEqual(INTERNAL_IP, ip_1)
        ip_2 = ctx.get_instance_ip(yarn.RESOURCE_MANAGER.ui_name)
        self.assertEqual(INTERNAL_IP, ip_2)
        none_ip = ctx.get_instance_ip(self.fake_np)
        self.assertIsNone(none_ip)

    def test_get_zookeeper_nodes_ip_with_port(self):
        ctx = self._get_context()

        expected = '%s:5181' % INTERNAL_IP
        actual = ctx.get_zookeeper_nodes_ip_with_port()
        self.assertEqual(expected, actual)

        management.ZK_CLIENT_PORT = '0000'
        expected = '%s:0000' % INTERNAL_IP
        actual = ctx.get_zookeeper_nodes_ip_with_port()
        self.assertEqual(expected, actual)

    def test_filter_instances(self):
        ctx = self._get_context()
        instances = ctx.get_instances()
        rsmngs = ctx.filter_instances(instances, yarn.RESOURCE_MANAGER)
        self.assertEqual(1, len(rsmngs))
        not_existing_i = ctx.filter_instances(instances, self.fake_np)
        self.assertEqual(0, len(not_existing_i))

    def test_check_for_process(self):
        ctx = self._get_context()
        instance = ctx.get_instance(yarn.RESOURCE_MANAGER)
        self.assertTrue(ctx.check_for_process(instance, yarn.RESOURCE_MANAGER))
        self.assertTrue(ctx.check_for_process(instance,
                                              yarn.RESOURCE_MANAGER.ui_name))
        self.assertFalse(ctx.check_for_process(instance, maprfs.NFS))
        self.assertFalse(ctx.check_for_process(instance, maprfs.NFS.ui_name))

    def test_get_chosen_service_version(self):
        ctx = self._get_context()
        version = ctx.get_chosen_service_version('Service')
        self.assertEqual('1.1', version)

    def test_get_cluster_services(self):
        ctx = self._get_context()
        actual_services = ctx.get_cluster_services()
        actual_services_names = map(lambda s: s.ui_name, actual_services)
        expected_services_names = [
            yarn.YARN().ui_name,
            management.Management().ui_name,
            maprfs.MapRFS().ui_name,
            oozie.Oozie().ui_name,
            swift.Swift().ui_name,
        ]
        self.assertEqual(sorted(actual_services_names),
                         sorted(expected_services_names))

    def test_get_service(self):
        ctx = self._get_context()
        service = ctx.get_service(yarn.HISTORY_SERVER)
        self.assertEqual(yarn.YARN().ui_name, service.ui_name)
        with testtools.ExpectedException(e.PluginInvalidDataException):
            ctx.get_service(self.fake_np)

    def test_get_service_name_by_node_process(self):
        ctx = self._get_context()
        s_name_1 = ctx.get_service_name_by_node_process(yarn.RESOURCE_MANAGER)
        self.assertEqual(yarn.YARN().ui_name, s_name_1)
        s_name_2 = ctx.get_service_name_by_node_process(
            yarn.RESOURCE_MANAGER.ui_name)
        self.assertEqual(yarn.YARN().ui_name, s_name_2)
        not_existing_np = np.NodeProcess('not_existing', 'NotExisting', 'foo')
        self.assertIsNone(ctx.get_service_name_by_node_process(
            not_existing_np))
        self.assertIsNone(ctx.get_service_name_by_node_process(
            not_existing_np.ui_name))

    def test_get_instances_count(self):
        ctx = self._get_context()
        self.assertEqual(1, ctx.get_instances_count())
        self.assertEqual(1, ctx.get_instances_count(yarn.RESOURCE_MANAGER))
        self.assertEqual(1, ctx.get_instances_count(
            yarn.RESOURCE_MANAGER.ui_name))
        self.assertEqual(0, ctx.get_instances_count(self.fake_np))
        self.assertEqual(0, ctx.get_instances_count(
            self.fake_np.ui_name))

    def test_get_node_groups(self):
        ctx = self._get_context()
        all_ngs = ctx.get_node_groups()
        self.assertEqual(1, len(all_ngs))
        self.assertEqual([self.ng], all_ngs)
        rm_ngs_1 = ctx.get_node_groups(yarn.RESOURCE_MANAGER)
        self.assertEqual(1, len(rm_ngs_1))
        self.assertEqual([self.ng], rm_ngs_1)
        rm_ngs_2 = ctx.get_node_groups(yarn.RESOURCE_MANAGER.ui_name)
        self.assertEqual(1, len(rm_ngs_2))
        self.assertEqual([self.ng], rm_ngs_2)
        empty_ngs = ctx.get_node_groups(self.fake_np)
        self.assertEqual(0, len(empty_ngs))

    def test_get_cldb_nodes_ip(self):
        ctx = self._get_context()
        cldb_list_1 = ctx.get_cldb_nodes_ip()
        self.assertEqual(1, len(cldb_list_1.split(',')))
        self.assertIn(INTERNAL_IP, cldb_list_1)
        cldb_list_2 = ctx.get_cldb_nodes_ip()
        self.assertEqual(1, len(cldb_list_2.split(',')))
        self.assertIn(INTERNAL_IP, cldb_list_2)
        sep = ':'
        cldb_list_3 = ctx.get_cldb_nodes_ip(sep)
        self.assertEqual(1, len(cldb_list_3.split(sep)))
        self.assertIn(INTERNAL_IP, cldb_list_3)

    def test_get_zookeeper_nodes_ip(self):
        ctx = self._get_context()
        zk_list_1 = ctx.get_zookeeper_nodes_ip()
        self.assertEqual(1, len(zk_list_1.split(',')))
        self.assertIn(INTERNAL_IP, zk_list_1)
        zk_list_2 = ctx.get_zookeeper_nodes_ip()
        self.assertEqual(1, len(zk_list_2.split(',')))
        self.assertIn(INTERNAL_IP, zk_list_2)
        sep = ':'
        zk_list_3 = ctx.get_zookeeper_nodes_ip(sep)
        self.assertEqual(1, len(zk_list_3.split(sep)))
        self.assertIn(INTERNAL_IP, zk_list_3)

    def test_get_resourcemanager_ip(self):
        ctx = self._get_context()
        ip = ctx.get_resourcemanager_ip()
        self.assertEqual(INTERNAL_IP, ip)

    def test_get_historyserver_ip(self):
        ctx = self._get_context()
        self.assertTrue(ctx.has_control_nodes([self.instance]))

    def test_is_present(self):
        cluster_context = self._get_context()

        self.assertTrue(cluster_context.is_present(oozie.Oozie()))
        self.assertFalse(cluster_context.is_present(oozie.OozieV401()))
        self.assertTrue(cluster_context.is_present(oozie.OozieV420()))



# Copyright (c) 2015, MapR Technologies
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#       http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.


import os

from oslo_utils import uuidutils

import sahara_plugin_mapr.plugins.mapr.util.general as g

MV_TO_MAPRFS_CMD = ('sudo -u %(user)s'
                    ' hadoop fs -copyFromLocal %(source)s %(target)s'
                    ' && sudo rm -f %(source)s')
CP_TO_MAPRFS_CMD = ('sudo -u %(user)s'
                    ' hadoop fs -copyFromLocal %(source)s %(target)s')

CP_FROM_MAPRFS_CMD = ('sudo hadoop fs -copyToLocal %(source)s %(target)s')

RM_CMD = ('sudo -u %(user)s hadoop fs -rm %(path)s')
MKDIR_CMD_MAPR4 = 'sudo -u %(user)s hadoop fs -mkdir -p %(path)s'
MKDIR_CMD_MAPR3 = 'sudo -u %(user)s hadoop fs -mkdir %(path)s'


def put_file_to_maprfs(r, content, file_name, path, hdfs_user):
    tmp_file_name = '/tmp/%s.%s' % (file_name, uuidutils.generate_uuid())
    r.write_file_to(tmp_file_name, content)
    target = os.path.join(path, file_name)
    move_from_local(r, tmp_file_name, target, hdfs_user)


def move_from_local(r, source, target, hdfs_user):
    args = {'user': hdfs_user, 'source': source, 'target': target}
    r.execute_command(MV_TO_MAPRFS_CMD % args)


def copy_from_local(r, source, target, hdfs_user):
    args = {'user': hdfs_user, 'source': source, 'target': target}
    r.execute_command(CP_TO_MAPRFS_CMD % args)


def copy_to_local(r, hdfs_source, target, hdfs_user, overwrite=True):
    args = {'source': hdfs_source, 'target': target}
    if overwrite:
        _rm_from_local(r, target)
    r.execute_command(CP_FROM_MAPRFS_CMD % args)


def exchange(source, target, src_path, trg_path, hdfs_user, alias_dir="/"):
    # check if target path contains file name, if not set file same as source
    if trg_path[-1] == '/':
        trg_path = trg_path + src_path.rsplit('/', 1)[1]
    alias = alias_dir + _generate_file_name()
    copy_from_local(source, src_path, alias, hdfs_user)
    copy_to_local(target, alias, trg_path, hdfs_user)
    remove(source, alias, hdfs_user)


def remove(r, path, hdfs_user):
    args = {'user': hdfs_user, 'path': path}
    r.execute_command(RM_CMD % args)


def create_maprfs4_dir(remote, dir_name, hdfs_user):
    remote.execute_command(MKDIR_CMD_MAPR4 % {'user': hdfs_user,
                                              'path': dir_name})


def create_maprfs3_dir(remote, dir_name, hdfs_user):
    remote.execute_command(MKDIR_CMD_MAPR3 % {'user': hdfs_user,
                                              'path': dir_name})


def mkdir(remote, path, recursive=True, run_as=None):
    command = 'hadoop fs -mkdir %(recursive)s %(path)s'
    args = {'recursive': '-p' if recursive else '', 'path': path}
    remote.execute_command(g._run_as(run_as, command % args))


def chmod(remote, path, mode, recursive=True, run_as=None):
    command = 'hadoop fs -chmod %(recursive)s %(mode)s %(path)s'
    args = {'recursive': '-R' if recursive else '', 'path': path, 'mode': mode}
    remote.execute_command(g._run_as(run_as, command % args))


def _generate_file_name(length=10):
    return uuidutils.generate_uuid()[:length]


def _rm_from_local(remote, file_name, exist=False):
    if exist:
        remote.execute_command("rm %s" % file_name, run_as_root=True)
    else:
        out, result = remote.execute_command("ls %s" % file_name,
                                             run_as_root=True,
                                             raise_when_error=False)
        if out == 0:
            remote.execute_command("rm %s" % file_name, run_as_root=True)



# Copyright (c) 2015, MapR Technologies
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#       http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.


import sahara_plugin_mapr.plugins.mapr.domain.node_process as np
import sahara_plugin_mapr.plugins.mapr.domain.service as s
import sahara_plugin_mapr.plugins.mapr.util.password_utils as pu
import sahara_plugin_mapr.plugins.mapr.util.validation_utils as vu

ZK_CLIENT_PORT = 5181

ZOOKEEPER = np.NodeProcess(
    name='mapr-zookeeper',
    ui_name='ZooKeeper',
    package='mapr-zookeeper',
    open_ports=[ZK_CLIENT_PORT]
)

WEB_SERVER = np.NodeProcess(
    name='webserver',
    ui_name='Webserver',
    package='mapr-webserver',
    open_ports=[8443]
)

METRICS = np.NodeProcess(
    name='metrics',
    ui_name='Metrics',
    package='mapr-metrics',
    open_ports=[1111]
)


class Management(s.Service):
    def __init__(self):
        super(Management, self).__init__()
        self._ui_name = 'Management'
        self._node_processes = [ZOOKEEPER, WEB_SERVER, METRICS]

        self._ui_info = None
        self._validation_rules = [
            vu.at_least(1, ZOOKEEPER),
            vu.at_least(1, WEB_SERVER),
            vu.odd_count_of(ZOOKEEPER),
        ]

    def get_ui_info(self, cluster_context):
        # MCS uses credentials of the administrative user (PAM auth)
        return [('MapR Control System (MCS)', WEB_SERVER,
                 {s.SERVICE_UI: 'https://%s:8443',
                  'Username': 'mapr',
                  'Password': pu.get_mapr_password(cluster_context.cluster)})]



# Copyright (c) 2015, MapR Technologies
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#       http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.


import sahara_plugin_mapr.plugins.mapr.domain.node_process as np
import sahara_plugin_mapr.plugins.mapr.domain.service as s
import sahara_plugin_mapr.plugins.mapr.util.validation_utils as vu

PIG = np.NodeProcess(
    name='pig',
    ui_name='Pig',
    package='mapr-pig'
)


class Pig(s.Service):
    def __init__(self):
        super(Pig, self).__init__()
        self._name = 'pig'
        self._ui_name = 'Pig'
        self._node_processes = [PIG]
        self._validation_rules = [vu.at_least(1, PIG)]


class PigV013(Pig):
    def __init__(self):
        super(PigV013, self).__init__()
        self._version = '0.13'


class PigV014(Pig):
    def __init__(self):
        super(PigV014, self).__init__()
        self._version = '0.14'


class PigV015(Pig):
    def __init__(self):
        super(PigV015, self).__init__()
        self._version = '0.15'


class PigV016(Pig):
    def __init__(self):
        super(PigV016, self).__init__()
        self._version = '0.16'



# Copyright (c) 2015, MapR Technologies
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#       http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.


import abc

from oslo_log import log as logging
import six

from sahara.plugins import conductor
from sahara.plugins import context
import sahara.plugins.utils as utils
from sahara_plugin_mapr.i18n import _
import sahara_plugin_mapr.plugins.mapr.abstract.configurer as ac
from sahara_plugin_mapr.plugins.mapr.domain import distro as d
from sahara_plugin_mapr.plugins.mapr.domain import service as srvc
import sahara_plugin_mapr.plugins.mapr.services.management.management as mng
import sahara_plugin_mapr.plugins.mapr.services.mapreduce.mapreduce as mr
from sahara_plugin_mapr.plugins.mapr.services.maprfs import maprfs
from sahara_plugin_mapr.plugins.mapr.services.mysql import mysql
import sahara_plugin_mapr.plugins.mapr.services.yarn.yarn as yarn
from sahara_plugin_mapr.plugins.mapr.util import event_log as el
import sahara_plugin_mapr.plugins.mapr.util.general as util
import sahara_plugin_mapr.plugins.mapr.util.password_utils as pu

LOG = logging.getLogger(__name__)

_JAVA_HOME = '/usr/java/jdk1.7.0_51'
_CONFIGURE_SH_TIMEOUT = 600
_SET_MODE_CMD = 'maprcli cluster mapreduce set -mode '

_TOPO_SCRIPT = 'plugins/mapr/resources/topology.sh'
INSTALL_JAVA_SCRIPT = 'plugins/mapr/resources/install_java.sh'
INSTALL_SCALA_SCRIPT = 'plugins/mapr/resources/install_scala.sh'
INSTALL_MYSQL_CLIENT = 'plugins/mapr/resources/install_mysql_client.sh'
ADD_MAPR_REPO_SCRIPT = 'plugins/mapr/resources/add_mapr_repo.sh'
ADD_SECURITY_REPO_SCRIPT = 'plugins/mapr/resources/add_security_repos.sh'

SERVICE_INSTALL_PRIORITY = [
    mng.Management(),
    yarn.YARNv251(),
    yarn.YARNv241(),
    yarn.YARNv270(),
    mr.MapReduce(),
    maprfs.MapRFS(),
]


@six.add_metaclass(abc.ABCMeta)
class BaseConfigurer(ac.AbstractConfigurer):
    def configure(self, cluster_context, instances=None):
        instances = instances or cluster_context.get_instances()
        self._configure_ssh_connection(cluster_context, instances)
        self._install_mapr_repo(cluster_context, instances)
        if not cluster_context.is_prebuilt:
            self._prepare_bare_image(cluster_context, instances)
        self._install_services(cluster_context, instances)
        if cluster_context.is_node_aware:
            self._configure_topology(cluster_context, instances)
        self._configure_database(cluster_context, instances)
        self._configure_services(cluster_context, instances)
        self._configure_sh_cluster(cluster_context, instances)
        self._set_cluster_mode(cluster_context, instances)
        self._post_configure_services(cluster_context, instances)
        self._write_config_files(cluster_context, instances)
        self._configure_environment(cluster_context, instances)
        self._update_cluster_info(cluster_context)

    def update(self, cluster_context, instances=None):
        LOG.debug('Configuring existing instances')
        instances = instances or cluster_context.get_instances()
        existing = cluster_context.existing_instances()
        if cluster_context.is_node_aware:
            self._configure_topology(cluster_context, existing)
        if cluster_context.has_control_nodes(instances):
            self._configure_sh_cluster(cluster_context, existing)
            self._post_configure_sh(cluster_context, existing)
        self._write_config_files(cluster_context, existing)
        self._update_services(cluster_context, existing)
        self._restart_services(cluster_context)
        self._update_cluster_info(cluster_context)
        LOG.info('Existing instances successfully configured')

    def _configure_services(self, cluster_context, instances):
        for service in cluster_context.cluster_services:
            service.configure(cluster_context, instances)

    def _install_services(self, cluster_context, instances):
        for service in self._service_install_sequence(cluster_context):
            service.install(cluster_context, instances)

    def _service_install_sequence(self, cluster_context):
        def key(service):
            if service in SERVICE_INSTALL_PRIORITY:
                return SERVICE_INSTALL_PRIORITY.index(service)
            return -service._priority

        return sorted(cluster_context.cluster_services, key=key, reverse=True)

    def _prepare_bare_image(self, cluster_context, instances):
        LOG.debug('Preparing bare image')
        if d.UBUNTU == cluster_context.distro:
            self._install_security_repos(cluster_context, instances)
        self._install_java(cluster_context, instances)
        self._install_scala(cluster_context, instances)
        self._install_mysql_client(cluster_context, instances)
        LOG.debug('Bare images successfully prepared')

    @el.provision_step(_("Install security repos"))
    def _install_security_repos(self, cluster_context, instances):
        LOG.debug("Installing security repos")

        @el.provision_event()
        def install_security_repos(instance):
            return util.run_script(instance, ADD_SECURITY_REPO_SCRIPT, "root")

        util.execute_on_instances(instances, install_security_repos)

    @el.provision_step(_("Install MySQL client"))
    def _install_mysql_client(self, cluster_context, instances):
        LOG.debug("Installing MySQL client")
        distro_name = cluster_context.distro.name

        @el.provision_event()
        def install_mysql_client(instance):
            return util.run_script(instance, INSTALL_MYSQL_CLIENT,
                                   "root", distro_name)

        util.execute_on_instances(instances, install_mysql_client)

    @el.provision_step(_("Install Scala"))
    def _install_scala(self, cluster_context, instances):
        LOG.debug("Installing Scala")
        distro_name = cluster_context.distro.name

        @el.provision_event()
        def install_scala(instance):
            return util.run_script(instance, INSTALL_SCALA_SCRIPT,
                                   "root", distro_name)

        util.execute_on_instances(instances, install_scala)

    @el.provision_step(_("Install Java"))
    def _install_java(self, cluster_context, instances):
        LOG.debug("Installing Java")
        distro_name = cluster_context.distro.name

        @el.provision_event()
        def install_java(instance):
            return util.run_script(instance, INSTALL_JAVA_SCRIPT,
                                   "root", distro_name)

        util.execute_on_instances(instances, install_java)

    @el.provision_step(_("Configure cluster topology"))
    def _configure_topology(self, cluster_context, instances):
        LOG.debug("Configuring cluster topology")

        topology_map = cluster_context.topology_map
        topology_map = ("%s %s" % item for item in six.iteritems(topology_map))
        topology_map = "\n".join(topology_map) + "\n"

        data_path = "%s/topology.data" % cluster_context.mapr_home
        script = utils.get_file_text(_TOPO_SCRIPT, 'sahara_plugin_mapr')
        script_path = '%s/topology.sh' % cluster_context.mapr_home

        @el.provision_event()
        def write_topology_data(instance):
            util.write_file(instance, data_path, topology_map, owner="root")
            util.write_file(instance, script_path, script,
                            mode="+x", owner="root")

        util.execute_on_instances(instances, write_topology_data)

        LOG.info('Cluster topology successfully configured')

    @el.provision_step(_("Write config files to instances"))
    def _write_config_files(self, cluster_context, instances):
        LOG.debug('Writing config files')

        @el.provision_event()
        def write_config_files(instance, config_files):
            for file in config_files:
                util.write_file(instance, file.path, file.data, mode=file.mode,
                                owner="mapr")

        node_groups = util.unique_list(instances, lambda i: i.node_group)
        for node_group in node_groups:
            config_files = cluster_context.get_config_files(node_group)
            ng_instances = [i for i in node_group.instances if i in instances]
            util.execute_on_instances(ng_instances, write_config_files,
                                      config_files=config_files)

        LOG.debug("Config files are successfully written")

    def _configure_environment(self, cluster_context, instances):
        self.configure_general_environment(cluster_context, instances)
        self._post_install_services(cluster_context, instances)

    def _configure_database(self, cluster_context, instances):
        mysql_instance = mysql.MySQL.get_db_instance(cluster_context)

        @el.provision_event(instance=mysql_instance,
                            name=_("Configure database"))
        def decorated():
            distro_name = cluster_context.distro.name
            distro_version = cluster_context.distro_version
            mysql.MySQL.install_mysql(mysql_instance, distro_name,
                                      distro_version)
            mysql.MySQL.start_mysql_server(cluster_context)
            mysql.MySQL.create_databases(cluster_context, instances)

        decorated()

    def _post_install_services(self, cluster_context, instances):
        LOG.debug('Executing service post install hooks')
        for s in cluster_context.cluster_services:
            service_instances = cluster_context.filter_instances(instances,
                                                                 service=s)
            if service_instances:
                s.post_install(cluster_context, instances)
        LOG.info('Post install hooks execution successfully executed')

    def _update_cluster_info(self, cluster_context):
        LOG.debug('Updating UI information.')
        info = {'Admin user credentials': {'Username': 'mapr',
                                           'Password': pu.get_mapr_password
                                           (cluster_context.cluster)}}
        for service in cluster_context.cluster_services:
            for title, node_process, ui_info in (
                    service.get_ui_info(cluster_context)):
                removed = cluster_context.removed_instances(node_process)
                instances = cluster_context.get_instances(node_process)
                instances = [i for i in instances if i not in removed]

                if len(instances) == 1:
                    display_name_template = "%(title)s"
                else:
                    display_name_template = "%(title)s %(index)s"

                for index, instance in enumerate(instances, start=1):
                    args = {"title": title, "index": index}
                    display_name = display_name_template % args
                    data = ui_info.copy()
                    data[srvc.SERVICE_UI] = (data[srvc.SERVICE_UI] %
                                             instance.get_ip_or_dns_name())
                    info.update({display_name: data})

        ctx = context.ctx()
        conductor.cluster_update(ctx, cluster_context.cluster, {'info': info})

    def configure_general_environment(self, cluster_context, instances=None):
        LOG.debug('Executing post configure hooks')
        mapr_user_pass = pu.get_mapr_password(cluster_context.cluster)

        if not instances:
            instances = cluster_context.get_instances()

        def set_user_password(instance):
            LOG.debug('Setting password for user "mapr"')
            if self.mapr_user_exists(instance):
                with instance.remote() as r:
                    r.execute_command(
                        'echo "%s:%s"|chpasswd' %
                        ('mapr', mapr_user_pass),
                        run_as_root=True)
            else:
                LOG.warning('User "mapr" does not exists')

        def create_home_mapr(instance):
            target_path = '/home/mapr'
            LOG.debug("Creating home directory for user 'mapr'")
            args = {'path': target_path,
                    'user': 'mapr',
                    'group': 'mapr'}
            cmd = ('mkdir -p %(path)s && chown %(user)s:%(group)s %(path)s'
                   % args)
            if self.mapr_user_exists(instance):
                with instance.remote() as r:
                    r.execute_command(cmd, run_as_root=True)
            else:
                LOG.warning('User "mapr" does not exists')

        util.execute_on_instances(instances, set_user_password)
        util.execute_on_instances(instances, create_home_mapr)

    @el.provision_step(_("Execute configure.sh"))
    def _configure_sh_cluster(self, cluster_context, instances):
        LOG.debug('Executing configure.sh')

        if not instances:
            instances = cluster_context.get_instances()
        script = cluster_context.configure_sh

        db_specs = dict(mysql.MySQL.METRICS_SPECS._asdict())
        db_specs.update({
            'host': mysql.MySQL.get_db_instance(cluster_context).internal_ip,
            'port': mysql.MySQL.MYSQL_SERVER_PORT,
        })

        with context.PluginsThreadGroup() as tg:
            for instance in instances:
                tg.spawn('configure-sh-%s' % instance.id,
                         self._configure_sh_instance, cluster_context,
                         instance, script, db_specs)
        LOG.debug('Executing configure.sh successfully completed')

    @el.provision_event(instance_reference=2)
    def _configure_sh_instance(self, cluster_context, instance, command,
                               specs):
        if not self.mapr_user_exists(instance):
            command += ' --create-user'
        if cluster_context.check_for_process(instance, mng.METRICS):
            command += (' -d %(host)s:%(port)s -du %(user)s -dp %(password)s '
                        '-ds %(db_name)s') % specs
        with instance.remote() as r:
            r.execute_command('sudo -i ' + command,
                              timeout=_CONFIGURE_SH_TIMEOUT)

    @el.provision_step(_("Configure SSH connection"))
    def _configure_ssh_connection(self, cluster_context, instances):
        @el.provision_event()
        def configure_ssh(instance):
            echo_param = 'echo "KeepAlive yes" >> ~/.ssh/config'
            echo_timeout = 'echo "ServerAliveInterval 60" >> ~/.ssh/config'
            with instance.remote() as r:
                r.execute_command(echo_param)
                r.execute_command(echo_timeout)

        util.execute_on_instances(instances, configure_ssh)

    def mapr_user_exists(self, instance):
        with instance.remote() as r:
            ec, __ = r.execute_command(
                "id -u %s" %
                'mapr', run_as_root=True, raise_when_error=False)
        return ec == 0

    def post_start(self, cluster_context, instances=None):
        instances = instances or cluster_context.get_instances()
        LOG.debug('Executing service post start hooks')
        for service in cluster_context.cluster_services:
            updated = cluster_context.filter_instances(instances,
                                                       service=service)
            service.post_start(cluster_context, updated)
        LOG.info('Post start hooks successfully executed')

    @el.provision_step(_("Set cluster mode"))
    def _set_cluster_mode(self, cluster_context, instances):
        cluster_mode = cluster_context.cluster_mode
        if not cluster_mode:
            return

        command = "maprcli cluster mapreduce set -mode %s" % cluster_mode

        @el.provision_event()
        def set_cluster_mode(instance):
            return util.execute_command([instance], command,
                                        run_as='mapr')

        util.execute_on_instances(instances, set_cluster_mode)

    @el.provision_step(_("Install MapR repositories"))
    def _install_mapr_repo(self, cluster_context, instances):
        distro_name = cluster_context.distro.name

        @el.provision_event()
        def install_mapr_repos(instance):
            return util.run_script(instance, ADD_MAPR_REPO_SCRIPT, "root",
                                   distro_name, **cluster_context.mapr_repos)

        util.execute_on_instances(instances, install_mapr_repos)

    def _update_services(self, cluster_context, instances):
        for service in cluster_context.cluster_services:
            updated = cluster_context.filter_instances(instances,
                                                       service=service)
            service.update(cluster_context, updated)

    def _restart_services(self, cluster_context):
        restart = cluster_context.should_be_restarted
        for service, instances in six.iteritems(restart):
            service.restart(util.unique_list(instances))

    def _post_configure_sh(self, cluster_context, instances):
        LOG.debug('Executing post configure.sh hooks')
        for service in cluster_context.cluster_services:
            service.post_configure_sh(cluster_context, instances)
        LOG.info('Post configure.sh hooks successfully executed')

    def _post_configure_services(self, cluster_context, instances):
        for service in cluster_context.cluster_services:
            service.post_configure(cluster_context, instances)



# Copyright (c) 2015, MapR Technologies
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#       http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.


import functools as ft

import sahara.plugins.exceptions as e
from sahara.plugins import resource as r
from sahara.plugins import service_api as api
import sahara.plugins.utils as utils
from sahara_plugin_mapr.i18n import _


class LessThanCountException(e.InvalidComponentCountException):
    MESSAGE = _("Hadoop cluster should contain at least"
                " %(expected_count)d %(component)s component(s)."
                " Actual %(component)s count is %(actual_count)d")

    def __init__(self, component, expected_count, count):
        super(LessThanCountException, self).__init__(
            component, expected_count, count)
        args = {
            'expected_count': expected_count,
            'component': component,
            'actual_count': count,
        }
        self.message = LessThanCountException.MESSAGE % args


class MoreThanCountException(e.InvalidComponentCountException):
    MESSAGE = _("Hadoop cluster should contain at most"
                " %(expected_count)d %(component)s component(s)."
                " Actual %(component)s count is %(actual_count)d")

    def __init__(self, component, expected_count, count):
        super(MoreThanCountException, self).__init__(
            component, expected_count, count)
        args = {
            "expected_count": expected_count,
            "component": component,
            "actual_count": count,
        }
        self.message = MoreThanCountException.MESSAGE % args


class EvenCountException(e.SaharaPluginException):
    MESSAGE = _("Hadoop cluster should contain odd number of %(component)s"
                " but %(actual_count)s found.")

    def __init__(self, component, count):
        super(EvenCountException, self).__init__()
        args = {'component': component, 'actual_count': count}
        self.message = EvenCountException.MESSAGE % args


class NodeRequiredServiceMissingException(e.RequiredServiceMissingException):
    MISSING_MSG = _('Node "%(ng_name)s" is missing component %(component)s')
    REQUIRED_MSG = _('%(message)s, required by %(required_by)s')

    def __init__(self, service_name, ng_name, required_by=None):
        super(NodeRequiredServiceMissingException, self).__init__(
            service_name, required_by)
        args = {'ng_name': ng_name, 'component': service_name}
        self.message = (
            NodeRequiredServiceMissingException.MISSING_MSG % args)
        if required_by:
            args = {'message': self.message, 'required_by': required_by}
            self.message = (
                NodeRequiredServiceMissingException.REQUIRED_MSG % args)


class NodeServiceConflictException(e.SaharaPluginException):
    MESSAGE = _('%(service)s service cannot be installed alongside'
                ' %(package)s package')
    ERROR_CODE = "NODE_PROCESS_CONFLICT"

    def __init__(self, service_name, conflicted_package):
        super(NodeServiceConflictException, self).__init__()
        args = {
            'service': service_name,
            'package': conflicted_package,
        }
        self.message = NodeServiceConflictException.MESSAGE % args
        self.code = NodeServiceConflictException.ERROR_CODE


class NoVolumesException(e.SaharaPluginException):
    MESSAGE = _('%s must have at least 1 volume or ephemeral drive')
    ERROR_CODE = "NO_VOLUMES"

    def __init__(self, ng_name):
        super(NoVolumesException, self).__init__()
        self.message = NoVolumesException.MESSAGE % ng_name
        self.code = NoVolumesException.ERROR_CODE


class NotRequiredImageException(e.SaharaPluginException):
    MESSAGE = _('Service %(service)s requires %(os)s OS.'
                ' Use %(os)s image and add "%(os)s" tag to it.')
    ERROR_CODE = "INVALID_IMAGE"

    def __init__(self, service, os):
        super(NotRequiredImageException, self).__init__()
        self.message = NotRequiredImageException.MESSAGE % {'service': service,
                                                            'os': os}
        self.code = NotRequiredImageException.ERROR_CODE


def at_least(count, component):
    def validate(cluster_context, component, count):
        actual_count = cluster_context.get_instances_count(component)
        if not actual_count >= count:
            raise LessThanCountException(
                component.ui_name, count, actual_count)

    return ft.partial(validate, component=component, count=count)


def at_most(count, component):
    def validate(cluster_context, component, count):
        actual_count = cluster_context.get_instances_count(component)
        if actual_count > count:
            raise MoreThanCountException(
                component.ui_name, count, actual_count)

    return ft.partial(validate, component=component, count=count)


def exactly(count, component):
    def validate(cluster_context, component, count):
        actual_count = cluster_context.get_instances_count(component)
        if not actual_count == count:
            raise e.InvalidComponentCountException(
                component.ui_name, count, actual_count)

    return ft.partial(validate, component=component, count=count)


def each_node_has(component):
    def validate(cluster_context, component):
        for node_group in cluster_context.cluster.node_groups:
            if component.ui_name not in node_group.node_processes:
                raise NodeRequiredServiceMissingException(
                    component.ui_name, node_group.name)

    return ft.partial(validate, component=component)


def odd_count_of(component):
    def validate(cluster_context, component):
        actual_count = cluster_context.get_instances_count(component)
        if actual_count > 1 and actual_count % 2 == 0:
            raise EvenCountException(component.ui_name, actual_count)

    return ft.partial(validate, component=component)


def on_same_node(component, dependency):
    def validate(cluster_context, component, dependency):
        for ng in cluster_context.get_node_groups(component):
            if dependency.ui_name not in ng.node_processes:
                raise NodeRequiredServiceMissingException(
                    dependency.ui_name, ng.name, component.ui_name)

    return ft.partial(validate, component=component, dependency=dependency)


def depends_on(service, required_by):
    def validate(cluster_context, service, required_by):
        if not cluster_context.is_present(service):
            service_name = service.ui_name
            if service.version:
                service_name += " %s" % service.version

            raise e.RequiredServiceMissingException(
                service_name, required_by.ui_name)

    return ft.partial(validate, service=service, required_by=required_by)


def node_client_package_conflict_vr(components, client_component):
    def validate(cluster_context, components):
        for ng in cluster_context.get_node_groups():
            for c in components:
                nps = ng.node_processes
                if c in nps and client_component in nps:
                    raise NodeServiceConflictException(c, client_component)

    return ft.partial(validate, components=components)


def assert_present(service, cluster_context):
    if not cluster_context.is_present(service):
        raise e.RequiredServiceMissingException(service.ui_name)


def required_os(os, required_by):
    def validate(cluster_context, os, required_by):
        for ng in cluster_context.get_node_groups():
            nps = ng.node_processes
            for node_process in required_by.node_processes:
                if node_process.ui_name in nps:
                    image_id = (ng.image_id or
                                cluster_context.cluster.default_image_id)
                    if not image_has_tag(image_id, os):
                        raise NotRequiredImageException(required_by.ui_name,
                                                        os)

    return ft.partial(validate, os=os, required_by=required_by)


def create_fake_cluster(cluster, existing, additional):
    counts = existing.copy()
    counts.update(additional)

    def update_ng(node_group):
        ng_dict = node_group.to_dict()
        count = counts[node_group.id]
        ng_dict.update(dict(count=count))
        return r.create_node_group_resource(ng_dict)

    def need_upd(node_group):
        return node_group.id in counts and counts[node_group.id] > 0

    updated = list(map(update_ng, filter(need_upd, cluster.node_groups)))
    not_updated = list(
        filter(lambda ng: not need_upd(ng) and ng is not None,
               cluster.node_groups))
    cluster_dict = cluster.to_dict()
    cluster_dict.update({'node_groups': updated + not_updated})
    fake = r.create_cluster_resource(cluster_dict)
    return fake


def get_ephemeral(node_group):
    return utils.get_flavor(id=node_group.flavor_id).ephemeral


def has_volumes():
    def validate(cluster_context):
        for node_group in cluster_context.cluster.node_groups:
            if not (node_group.volumes_per_node or get_ephemeral(node_group)):
                raise NoVolumesException(node_group.name)

    return validate


def image_has_tag(image_id, tag):
    image = api.get_registered_image(image_id)
    return tag in image.tags



# Copyright (c) 2015, MapR Technologies
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#       http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.


import sahara_plugin_mapr.plugins.mapr.domain.node_process as np
import sahara_plugin_mapr.plugins.mapr.domain.service as s
import sahara_plugin_mapr.plugins.mapr.util.validation_utils as vu


FLUME = np.NodeProcess(
    name='flume',
    ui_name='Flume',
    package='mapr-flume',
    open_ports=[]
)


class Flume(s.Service):
    def __init__(self):
        super(Flume, self).__init__()
        self._name = 'flume'
        self._ui_name = 'Flume'
        self._node_processes = [FLUME]
        self._validation_rules = [vu.at_least(1, FLUME)]


class FlumeV15(Flume):
    def __init__(self):
        super(FlumeV15, self).__init__()
        self._version = '1.5.0'


class FlumeV16(Flume):
    def __init__(self):
        super(FlumeV16, self).__init__()
        self._version = '1.6.0'



# Copyright (c) 2015, MapR Technologies
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#       http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.


import sahara_plugin_mapr.plugins.mapr.base.base_cluster_context as bc
import sahara_plugin_mapr.plugins.mapr.services.yarn.yarn as yarn


class Context(bc.BaseClusterContext):
    def __init__(self, cluster, version_handler, added=None, removed=None):
        super(Context, self).__init__(cluster, version_handler, added, removed)
        self._hadoop_version = yarn.YARNv270().version
        self._hadoop_lib = None
        self._hadoop_conf = None
        self._cluster_mode = yarn.YARNv270.cluster_mode
        self._node_aware = True
        self._resource_manager_uri = "maprfs:///"
        self._mapr_version = "5.2.0"
        self._ubuntu_ecosystem_repo = (
            "http://package.mapr.com/releases/MEP/MEP-2.0.0/ubuntu/"
            " binary trusty")
        self._centos_ecosystem_repo = (
            "http://package.mapr.com/releases/MEP/MEP-2.0.0/redhat")

    @property
    def hadoop_lib(self):
        if not self._hadoop_lib:
            self._hadoop_lib = "%s/share/hadoop/common" % self.hadoop_home
        return self._hadoop_lib

    @property
    def hadoop_conf(self):
        if not self._hadoop_conf:
            self._hadoop_conf = "%s/etc/hadoop" % self.hadoop_home
        return self._hadoop_conf

    @property
    def resource_manager_uri(self):
        return self._resource_manager_uri

    @property
    def configure_sh(self):
        if not self._configure_sh:
            configure_sh_template = "%(base)s -HS %(history_server)s"
            args = {
                "base": super(Context, self).configure_sh,
                "history_server": self.get_historyserver_ip(),
            }
            self._configure_sh = configure_sh_template % args
        return self._configure_sh
